#if defined _ash_included
 #endinput
#endif
#define _ash_included

#define PLUGIN_NAME_RESERVED_LENGTH 33

enum ASH_LogLevel {
	ASH_LogLevelNone = 0,  /**< Logging disabled */
	ASH_LogLevelBan,     /**< Log only bans */
	ASH_LogLevelSuspicious,     /**< Log suspicious players */
	ASH_LogLevelInfo,     /**< Log general infos */
	ASH_LogLevelAll,     /**< Log all players */
	ASH_LogLevelDebug     /**< Log debug infos */
};

/**
 * Get true if client is recording, false if not.
 *
 * @param client     Client ID.
 * @return bool 	status
 */
native bool:ASH_GetRecordingStatus(client);

/**
 * Reset strafe stats.
 *
 * @param client     Client ID.
 * @noreturn
 */
native ASH_ResetStrafes(client);

/**
 * Start recording strafes, also resets them.
 *
 * @param client     Client ID.
 * @noreturn
 */
native ASH_StartRecordingStrafes(client);

/**
 * Pause recording strafe stats.
 *
 * @param client     Client ID.
 * @noreturn
 */
native ASH_PauseRecordingStrafes(client);

/**
 * Resume recording strafe stats.
 *
 * @param client     Client ID.
 * @noreturn
 */
native ASH_ResumeRecordingStrafes(client);

/**
 * Compute strafe stats and take action (log/ban/etc.) depending on ASH cvar settings.
 *
 * @param client     Client ID.
 * @noreturn
 */
native ASH_ComputeStrafes(client);

/**
 * Returns the current logging level.
 *
 * @return              Current logging level
 */
native ASH_LogLevel:ASH_GetLogLevel();

/**
 * Logs a message to the ASH log file. Depending on the log level provided, this is equivalent
 * to calling any of the logging functions below (ASH_LogBan(), Timer_LogSuspicious(), etc).
 *
 * @param log_level     The severity of the message being logged.
 * @param format        A format string.
 * @param ...           Variable number of format parameters.
 * @noreturn
 */
native ASH_Log(ASH_LogLevel:log_level, const String:format[] , any:...);

/**
 * Logs a message to the ASH log file. Usually banned players.
 *
 * @param format        A format string.
 * @param ...           Variable number of format parameters.
 * @noreturn
 */
native ASH_LogBan(const String:format[] , any:...);

/**
 * Logs a message to the ASH log file. Usually suspicious players.
 *
 * @param format        A format string.
 * @param ...           Variable number of format parameters.
 * @noreturn
 */
native ASH_LogSuspicious(const String:format[] , any:...);

/**
 * Logs an message to the ASH log file. Usually general info.
 *
 * @param format        A format string.
 * @param ...           Variable number of format parameters.
 * @noreturn
 */
native ASH_LogInfo(const String:format[] , any:...);

/**
 * Logs a message to the ASH log file. All info you can get.
 *
 * @param format        A format string.
 * @param ...           Variable number of format parameters.
 * @noreturn
 */
native ASH_LogAll(const String:format[] , any:...);

/**
 * Logs a message to the ASH log file. All debug info you can get.
 *
 * @param format        A format string.
 * @param ...           Variable number of format parameters.
 * @noreturn
 */
native ASH_LogDebug(const String:format[] , any:...);

/** Forwards:
* client - client id
* fPerfect - percentage of 0-tick strafes
* fVeryGood - percentage of 1-tick strafes
* fGood - percentage of 2-tick strafes
* fPre - percentage of pre pressed strafes
* nStrafeCount - amount of analyzed strafes
*/

/* Just forwarding validated strafe stats */
forward ASH_OnClientComputed(client, Float:fPerfect, Float:fVeryGood, Float:fGood, Float:fPre, nStrafeCount);
/* Logged a player for suspicious strafe stats */
forward ASH_OnClientSuspicious(client, Float:fPerfect, Float:fVeryGood, Float:fGood, Float:fPre, nStrafeCount);
/* Banned a player for suspicious strafe stats */
forward ASH_OnClientBanned(client, Float:fPerfect, Float:fVeryGood, Float:fGood, Float:fPre, nStrafeCount);

public SharedPlugin:__pl_ash = 
{
	name = "anti-strafe-hack",
	file = "anti-strafe-hack.smx",
#if defined REQUIRE_PLUGIN
	required = 1,
#else
	required = 0,
#endif
};

public __pl_ash_SetNTVOpt() 
{
	MarkNativeAsOptional("ASH_GetRecordingStatus");
	
	MarkNativeAsOptional("ASH_ResetStrafes");
	MarkNativeAsOptional("ASH_StartRecordingStrafes");
	MarkNativeAsOptional("ASH_PauseRecordingStrafes");
	MarkNativeAsOptional("ASH_ResumeRecordingStrafes");
	MarkNativeAsOptional("ASH_ComputeStrafes");
	MarkNativeAsOptional("ASH_GetLogLevel");
	
	MarkNativeAsOptional("ASH_Log");
	MarkNativeAsOptional("ASH_LogBan");
	MarkNativeAsOptional("ASH_LogSuspicious");
	MarkNativeAsOptional("ASH_LogInfo");
	MarkNativeAsOptional("ASH_LogAll");
	MarkNativeAsOptional("ASH_LogDebug");
}