/*
include/ss_checks.sp
SStocks
This plugin is coded by Alican "AlicanC" ubukuolu (alicancubukcuoglu@gmail.com)
Copyright (C) 2007 Alican ubukuolu
*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#if defined _ss_checks_included
 #endinput
#endif
#define _ss_checks_included

/**
 * Checks the modification.
 *
 * @param mod			Modification name.
 * @return			True if mod matches, false otherwise.
 */
stock bool:CheckMod(const String:mod[])
	{
	decl String:modname[64];
	GetGameFolderName(modname, sizeof(modname));
	if(StrEqual(modname, mod))
		return true;
	return false;
	}

/**
 * Checks a plugin using common version ConVars.
 *
 * @param pname			Plugin's name.
 * @return			True if plugin is found, false otherwise.
 */
stock bool:CheckPlugin(const String:pname[])
	{
	new String:pversioncv[32];
	Format(pversioncv, sizeof(pversioncv), "%s_version", pname);
	if(FindConVar(pversioncv)!=INVALID_HANDLE)
		{
		return true;
		}
		else
		{
		Format(pversioncv, sizeof(pversioncv), "sm_%s_version", pname);
		return (FindConVar(pversioncv)!=INVALID_HANDLE);
		}
	}

/**
 * Checks a file for existence.
 *
 * @param file_path		Path to file from 'sourcemod'.
 * @return			True if file is found, false otherwise.
 */
stock bool:CheckFile(const String:file_path[])
	{
	new String:path[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, path, PLATFORM_MAX_PATH, file_path);
	if(FileExists(path))
		return true;
	return false;
	}

/**
 * Checks a required file for existence.
 * Sets a fail state if file doesn't exist.
 *
 * @param file_path		Path to file from 'sourcemod'.
 * @param pname			Plugin's name.
 * @return			True if file is found, false otherwise.
 */
stock bool:CheckRequiredFile(const String:file_path[], const String:pname[])
	{
	new String:path[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, path, PLATFORM_MAX_PATH, file_path);
	if(FileExists(path))
		return true;
	Fail(pname, "[%s] Required file '%s' not found.", pname, file_path);
	return false;
	}

/**
 * Checks a directory for existence.
 *
 * @param directory_path	Path to directory from 'sourcemod'.
 * @return			True if directory is found, false otherwise.
 */
stock bool:CheckDirectory(const String:directory_path[])
	{
	new String:path[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, path, PLATFORM_MAX_PATH, directory_path);
	if(DirExists(path))
		return true;
	return false;
	}

/**
 * Checks a required directory for existence.
 * Sets a fail state if directory doesn't exist.
 *
 * @param directory_path		Path to file from 'sourcemod'.
 * @param pname			Plugin's name.
 * @return			True if directory is found, false otherwise.
 */
stock bool:CheckRequiredDirectory(const String:directory_path[], const String:pname[])
	{
	new String:path[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, path, PLATFORM_MAX_PATH, directory_path);
	if(DirExists(path))
		return true;
	Fail(pname, "[%s] Required directory '%s' not found.", pname, directory_path);
	return false;
	}