/*
include/ss_clients.sp
SStocks
This plugin is coded by Alican "AlicanC" ubukuolu (alicancubukcuoglu@gmail.com)
Copyright (C) 2007 Alican ubukuolu
*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#if defined _ss_clients_included
 #endinput
#endif
#define _ss_clients_included

/**
 * Modifies a client's entity data.
 *
 * @param client		Client.
 * @param property		A 'CCSPlayer' class property.
 * @param todo			What to do with the offset. Valid values are 'get', 'set', 'inc', 'dec'.
 * @param value			Value.
 * @return			Returns -1 on failure, the last value of the offset otherwise.
 */
stock ClientData(client, const String:property[], const String:todo[], value=0)
	{
	new offset= FindSendPropOffs("CCSPlayer", property);
	if(offset==-1)
		return -1;
	new currentvalue=GetEntData(client, offset);
	if(StrEqual(todo,"set"))
		SetEntData(client, offset, value);
	else if(StrEqual(todo,"inc"))
		SetEntData(client, offset, currentvalue+value);
	else if(StrEqual(todo,"dec"))
		SetEntData(client, offset, currentvalue-value);
	return GetEntData(client, offset);
	}

/**
 * Modifies a client's health.
 *
 * @type			Shortcut.
 * @param client		Client.
 * @param todo			What to do with client's health. Valid values are 'get', 'set', 'inc', 'dec'.
 * @param value			Value.
 * @return			Returns -1 on failure, client's health otherwise.
 */
stock ClientHealth(client, const String:todo[], value=0)
	{
	return ClientData(client, "m_iHealth", todo, value);
	}

/**
 * Modifies a client's money.
 *
 * @type			Shortcut.
 * @param client		Client.
 * @param todo			What to do with client's money. Valid values are 'get', 'set', 'inc', 'dec'.
 * @param value			Value.
 * @return			Returns -1 on failure, client's money otherwise.
 */
stock ClientMoney(client, const String:todo[], value=0)
	{
	return ClientData(client, "m_iAccount", todo, value);
	}

/**
 * Sets a client's speed.
 *
 * @type			Shortcut.
 * @param client		Client.
 * @param speed			Speed. '1.0' is the default speed, '2.0' means two times faster.
 * @noreturn
 */
stock SetClientSpeed(client, Float:speed)
	{
	SetEntPropFloat(client, Prop_Data, "m_flLaggedMovementValue", speed);
	}

/**
 * Checks a client's flag.
 * If client doesn't have the flag, client will see an 'Access Denied' message.
 *
 * @param client		Client.
 * @param flag			Admin flag.
 * @return			True if client has the flag, false otherwise.
 */
stock bool:HasClientFlag(client, AdminFlag:flag, AdmAccessMode:mode=Access_Effective)
	{
	new AdminId:aid= GetUserAdmin(client);
	if((aid!=INVALID_ADMIN_ID) && GetAdminFlag(aid, flag, mode))
		return true;
	PrintToChat(client, "%c[%cAccess Denied%c]%c You don't have permission to access that command!", COLOR_GREEN, COLOR_TEAMCOLOR, COLOR_GREEN, COLOR_YELLOW);
	return false;
	}

/**
 * Checks a client's flag.
 * If client doesn't have the flag, client will see the formatted message.
 *
 * @param client		Client.
 * @param flag			Admin flag.
 * @param pname			Plugin's name.
 * @param format		Formatting rules.
 * @param ...			Variable number of format parameters.
 * @return			True if client has the flag, false otherwise.
 */
stock bool:HasClientFlagExt(client, AdminFlag:flag, AdmAccessMode:mode=Access_Effective, const String:pname[], const String:format[], any:...)
	{
	new AdminId:aid= GetUserAdmin(client);
	if((aid!=INVALID_ADMIN_ID) && GetAdminFlag(aid, flag, mode))
		return true;
	new String:message[128];
	VFormat(message, sizeof(message), format, 6);
	PrintToChat(client, "%c[%s]%c %s", COLOR_GREEN, pname, COLOR_YELLOW, message);
	return false;
	}

/**
 * Counts clients in a team.
 *
 * @param teamindex		Index of the team to count.
 * @return			Client count in the team.
 */
 stock CountClientsInTeam(teamindex)
	{
	new count= 0;
	new clients= GetMaxClients();
	for(new client= 1; client<=clients; client++)
		{
		if(IsClientInGame(client) && GetClientTeam(client)==teamindex)
			count++;
		}
	return count;
	}

/**
 * Counts admins in the server.
 *
 * @thanksto			BAILOPAN
 *
 * @param teamindex		Index of the team to count.
 * @return			Client count in the team.
 */
 stock CountAdmins(teamindex)
	{
	new count= 0;
	new clients= GetMaxClients();
	for(new client= 1; client<=clients; client++)
		{
		if(IsClientInGame(client) && GetUserAdmin(client)!=INVALID_ADMIN_ID)
			count++;
		}
	return count;
	}