#include "roleplay/roleplay"

#define PL_NAME		"RP_BUILDING_Appartment_DOORS"

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

/* ADT ARRAY */
new nAppartmentDoorAccessTeamplate[RP_AppartmentAccess];
new Handle:g_hAppartmentDoorAccess = INVALID_HANDLE;

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_GetPlayerAppartmentAccess", Native_GetPlayerAppartmentAccess);
	CreateNative("RP_SetPlayerAppartmentAccess", Native_SetPlayerAppartmentAccess);
	
	return APLRes_Success;
}

public OnPluginStart()
{
	g_hAppartmentDoorAccess = CreateArray(sizeof(nAppartmentDoorAccessTeamplate));
	nAppartmentDoorAccessTeamplate[AppartmentAccess_PlayerID] = 0;
	
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
	
	decl String:sQuery[2048];
	
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_building_appartment_doors` (  `PlayerID` int(10) NOT NULL, `AppartmentID` int(10) NOT NULL, `AccessID` int(10) NOT NULL, PRIMARY KEY (`PlayerID`,`BuildingID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	ResetAppartmentDoorAccess();
	LoadAppartmentDoorAccess();
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

ResetAppartmentDoorAccess()
{
	ClearArray(g_hAppartmentDoorAccess);
	g_hAppartmentDoorAccess = CreateArray(sizeof(nAppartmentDoorAccessTeamplate));
}

LoadAppartmentDoorAccess()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `PlayerID`, `AccessID`, `AppartmentID` FROM `rp_building_Appartment_doors`;");
	SQL_TQuery(g_hSQL, SQL_LoadAppartmentDoorAccessCallback, sQuery, _, DBPrio_Low);
}

public SQL_LoadAppartmentDoorAccessCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadAppartmentDoorAccessCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_AppartmentAccess];
		
		nNew[AppartmentAccess_PlayerID] = SQL_FetchInt(hndl, 0);
		nNew[AppartmentAccess_AccessID] = SQL_FetchInt(hndl, 1);
		nNew[AppartmentAccess_AppartementID] = SQL_FetchInt(hndl, 2);
		
		PushArrayArray(g_hAppartmentDoorAccess, nNew[0]);
	}
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	//Database is now ready to use
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	//PlayerID is now cached
}

public RP_OnDoorRoundstart(doorID,type)
{
	if(type == Door_Appartment)
	{
		RP_UnlockDoor(doorID);
	}
}

public RP_OnDoorUse(doorID,type,client)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if(type == Door_Appartment)
	{
		
	}
}

GetPlayerAppartmentAccess(client, buildingID)
{
	if(0 >= client || client > MaxClients + 1)
		return -1;
	
	if(!IsClientInGame(client))
		return -1;
	
	if(!IsPlayerAlive(client))
		return -1;
	
	new playerID = RP_GetPlayerID(client);
	
	for (new i = 0; i < GetArraySize(g_hAppartmentDoorAccess); i++)
	{
		new nNew[RP_AppartmentAccess];
		GetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]);
		
		if (nNew[AppartmentAccess_PlayerID] == playerID && nNew[BankAccess_BuildingID] == buildingID)
		{
			return nNew[AppartmentAccess_AccessID];
		}
	}
	
	return -1;

}

SetPlayerAppartmentAccess(client, buildingID, appartmentID, new_accessID)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	new playerID = RP_GetPlayerID(client);
	new bool:insert = true;
	
	for (new i = 0; i < GetArraySize(g_hAppartmentDoorAccess); i++)
	{
		new nNew[RP_AppartmentAccess];
		GetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]);
		
		if (nNew[AppartmentAccess_PlayerID] == playerID && nNew[BankAccess_BuildingID] == buildingID)
		{
			nNew[AppartmentAccess_AccessID] = new_accessID;
			SetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]); 
			insert = false;
		}
	}
	
	if(insert)
	{
		decl String:sQuery[2048];
	
		Format(sQuery, sizeof(sQuery), "INSERT INTO `rp_building_Appartment_doors` ( PlayerID, BuildingID, AppartmentID , AccessID) VALUES (%d, %d, %d, %d);",playerID, buildingID,appartmentID,new_accessID);
		LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
		SQL_TQuery(g_hSQL, SQL_InsertPlayerAppartmentAccessTableCallback, sQuery, _, DBPrio_High);
	}
}

public SQL_InsertPlayerAppartmentAccessTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_InsertPlayerAppartmentAccessTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	ResetAppartmentDoorAccess();
	LoadAppartmentDoorAccess();
}

public Native_GetPlayerAppartmentAccess(Handle:plugin, numParams)
{
	return GetPlayerAppartmentAccess(GetNativeCell(1),GetNativeCell(2));
}

public Native_SetPlayerAppartmentAccess(Handle:plugin, numParams)
{
	SetPlayerAppartmentAccess(GetNativeCell(1),GetNativeCell(2),GetNativeCell(3),GetNativeCell(4));
}