#include "roleplay/roleplay"

#define PL_NAME		"RP_BUILDING_BANK_DOORS"
#define DOOR_OPEN_TIME 4.0;

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

/* ADT ARRAY */
new nBankDoorAccessTeamplate[RP_BankAccess];
new Handle:g_hBankDoorAccess = INVALID_HANDLE;

new g_SelectedDoorID[MAXPLAYERS + 1];

new String:g_sBankAccessName[3][128] =  { "Bank Manager", "Bank Worker", "Customer"};

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_GetPlayerBankAccess", Native_GetPlayerBankAccess);
	CreateNative("RP_SetPlayerBankAccess", Native_SetPlayerBankAccess);
	
	return APLRes_Success;
}

public OnPluginStart()
{
	g_hBankDoorAccess = CreateArray(sizeof(nBankDoorAccessTeamplate));
	nBankDoorAccessTeamplate[BankAccess_PlayerID] = 0;
	
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
	
	decl String:sQuery[2048];
	
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_building_bank_doors` (  `PlayerID` int(10) NOT NULL, `BuildingID` int(10) NOT NULL, `AccessID` int(10) NOT NULL, PRIMARY KEY (`PlayerID`, `BuildingID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	ResetBankDoorAccess();
	LoadBankDoorAccess();
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

ResetBankDoorAccess()
{
	ClearArray(g_hBankDoorAccess);
	g_hBankDoorAccess = CreateArray(sizeof(nBankDoorAccessTeamplate));
}

LoadBankDoorAccess()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `PlayerID`, `AccessID`, `BuildingID` FROM `rp_building_bank_doors`;");
	SQL_TQuery(g_hSQL, SQL_LoadBankDoorAccessCallback, sQuery, _, DBPrio_Low);
}

public SQL_LoadBankDoorAccessCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadBankDoorAccessCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_BankAccess];
		
		nNew[BankAccess_PlayerID] = SQL_FetchInt(hndl, 0);
		nNew[BankAccess_AccessID] = SQL_FetchInt(hndl, 1);
		nNew[BankAccess_BuildingID] = SQL_FetchInt(hndl, 2);
		
		PushArrayArray(g_hBankDoorAccess, nNew[0]);
	}
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	//Database is now ready to use
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	//PlayerID is now cached
}

public RP_OnDoorUse(doorID,type,client)
{
	if(type == Door_Bank_Entrance || type == Door_Bank_WorkerEntrance || type == Door_Bank_Office || type ==  Door_Bank_ATMDoor)
	{
		if(0 >= client || client > MaxClients + 1)
			return;
	
		if(!IsClientInGame(client))
			return;
	
		if(!IsPlayerAlive(client))
			return;
	
		new buildingID = RP_GetDoorBuildingID(doorID);
		new accessID = GetPlayerBankAccess(client, buildingID);
		
		if(accessID == -1)
		{
			Menu_DoorBank(client, doorID, type, accessID);
		}
	}
}

Menu_DoorBank(client, doorID, type, accessID)
{	
	g_SelectedDoorID[client] = doorID;
	
	new Handle:menu = CreateMenu(MenuHandler_DoorBank);
	SetMenuTitle(menu, "Bank Door Menu: Accesslevel: %s",g_sBankAccessName[accessID]);
	
	if(accessID == Bank_Doors_Manager_Access || ( accessID == Bank_Doors_Worker_Access && (type == Door_Bank_Entrance || type == Door_Bank_WorkerEntrance || type ==  Door_Bank_ATMDoor)))
	{
		AddMenuItemEx(menu, false, "0", "%s Door", RP_IsDoorLocked(doorID) ? "Lock" : "Unlock");
		AddMenuItemEx(menu, false, "1", "Open Door");
	}
	else if(accessID == Bank_Doors_Costumer_Access && (type == Door_Bank_ATMDoor || type == Door_Bank_Entrance))
	{
		AddMenuItemEx(menu, false, "1", "Open Door");
	}
	
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_DoorBank(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new selection = StringToInt(info);
			
			if(selection == 0)
			{
				if(RP_IsDoorLocked(g_SelectedDoorID[client]))
					RP_UnlockDoor(g_SelectedDoorID[client]);
				else
					RP_LockDoor(g_SelectedDoorID[client]);
			}				
			else if(selection == 1)
			{
				RP_UnlockDoor(g_SelectedDoorID[client]);
				RP_OpenDoor(g_SelectedDoorID[client]);
				RP_LockDoor(g_SelectedDoorID[client]);
				CreateTimer(4.0, Timer_CloseDoor, g_SelectedDoorID[client], TIMER_REPEAT);
			}
			return;
		}
	}
}

public Action:Timer_CloseDoor(Handle:timer, any:doorID)
{
	RP_UnlockDoor(doorID);
	RP_CloseDoor(doorID);
	RP_LockDoor(doorID);
	return Plugin_Handled;
}

GetPlayerBankAccess(client, buildingID)
{
	if(0 >= client || client > MaxClients + 1)
		return -1;
	
	if(!IsClientInGame(client))
		return -1;
	
	if(!IsPlayerAlive(client))
		return -1;
	
	new playerID = RP_GetPlayerID(client);
	
	for (new i = 0; i < GetArraySize(g_hBankDoorAccess); i++)
	{
		new nNew[RP_BankAccess];
		GetArrayArray(g_hBankDoorAccess, i, nNew[0]);
		
		if (nNew[BankAccess_PlayerID] == playerID && nNew[BankAccess_BuildingID] == buildingID)
		{
			return nNew[BankAccess_AccessID];
		}
	}
	
	return -1;

}

SetPlayerBankAccess(client, buildingID, new_accessID)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	new playerID = RP_GetPlayerID(client);
	new bool:insert = true;
	
	for (new i = 0; i < GetArraySize(g_hBankDoorAccess); i++)
	{
		new nNew[RP_BankAccess];
		GetArrayArray(g_hBankDoorAccess, i, nNew[0]);
		
		if (nNew[BankAccess_PlayerID] == playerID && nNew[BankAccess_BuildingID] == buildingID)
		{
			nNew[BankAccess_AccessID] = new_accessID;
			SetArrayArray(g_hBankDoorAccess, i, nNew[0]); 
			insert = false;
		}
	}
	
	if(insert)
	{
		decl String:sQuery[2048];
	
		Format(sQuery, sizeof(sQuery), "INSERT INTO `rp_building_bank_doors` ( PlayerID, BuildingID , AccessID) VALUES (%d, %d, %d);",playerID, buildingID,new_accessID);
		LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
		SQL_TQuery(g_hSQL, SQL_InsertPlayerBankAccessTableCallback, sQuery, _, DBPrio_High);
	}
}

public SQL_InsertPlayerBankAccessTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_InsertPlayerBankAccessTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public Native_GetPlayerBankAccess(Handle:plugin, numParams)
{
	return GetPlayerBankAccess(GetNativeCell(1),GetNativeCell(2));
}

public Native_SetPlayerBankAccess(Handle:plugin, numParams)
{
	SetPlayerBankAccess(GetNativeCell(1),GetNativeCell(2),GetNativeCell(3));
}