#include "roleplay/roleplay"

#define PL_NAME		"RP_BUILDING_APPARTMENT_ACCESS"
#define MAXACCESSTYPES 3
#define MAX_NOTE_INFO_TIME 120.0
#define APPARTMENT_DOOR_INTERACTION_RANGE 125.0

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

/* ADT ARRAY */
new nAppartmentDoorAccessTeamplate[RP_AppartmentAccess];
new Handle:g_hAppartmentDoorAccess = INVALID_HANDLE;

/* Menu */
new g_iAppartmentSelection[MAXPLAYERS + 1];
new g_iDoorIDSelection[MAXPLAYERS + 1];
new g_iPlayerIDSelection[MAXPLAYERS + 1];
new Float:g_fLastNote[MAXPLAYERS+1];

new String:AccesTypeName[MAXACCESSTYPES][32] =  { "Owner", "Manager", "Normal"};

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_GetPlayerAppartmentPlayerCount", Native_GetPlayerAppartmentPlayerCount);
	CreateNative("RP_GetPlayerAppartmentAccess", Native_GetPlayerAppartmentAccess);
	CreateNative("RP_SetPlayerAppartmentAccess", Native_SetPlayerAppartmentAccess);
	
	return APLRes_Success;
}

public OnPluginStart()
{
	g_hAppartmentDoorAccess = CreateArray(sizeof(nAppartmentDoorAccessTeamplate));
	nAppartmentDoorAccessTeamplate[AppartmentAccess_PlayerID] = 0;
	
	RegConsoleCmd("sm_info", Command_Info);
	
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
}

public Action:Command_Info(client, args)
{
	if(0 >= client || client > MaxClients + 1)
		return Plugin_Handled;
	
	if(!IsClientInGame(client))
		return Plugin_Handled;
	
	if(!IsPlayerAlive(client))
		return Plugin_Handled;
	
	new entity = GetClientAimTarget(client, false);
	
	if(RP_IsValidDoorEnt(entity))
	{
		new doorID = RP_GetDoorIDbyEntity(entity);
		new buildingType = RP_GetBuildingType(RP_GetDoorBuildingID(doorID));
		
		if(buildingType == BD_AppartmentHouse)
		{		
			g_iDoorIDSelection[client] = doorID;
			g_iAppartmentSelection[client] = RP_GetDoorAppartmentID(doorID);
			if(g_iAppartmentSelection[client] >= 0)
				Menu_AppartmentDoorMain(client);
			else
				PrintToChat(client, "This door doesn't belong to a appartment yet");
		}
	}
	return Plugin_Handled;
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

ResetAppartmentDoorAccess()
{
	ClearArray(g_hAppartmentDoorAccess);
	g_hAppartmentDoorAccess = CreateArray(sizeof(nAppartmentDoorAccessTeamplate));
}

LoadAppartmentDoorAccess()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `playerID`, `accessID`, `appartmentID` FROM `rp_building_appartment_access`;");
	SQL_TQuery(g_hSQL, SQL_LoadAppartmentDoorAccessCallback, sQuery, _, DBPrio_Low);
}

public SQL_LoadAppartmentDoorAccessCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadAppartmentDoorAccessCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_AppartmentAccess];
		
		nNew[AppartmentAccess_PlayerID] = SQL_FetchInt(hndl, 0);
		nNew[AppartmentAccess_AccessID] = SQL_FetchInt(hndl, 1);
		nNew[AppartmentAccess_AppartementID] = SQL_FetchInt(hndl, 2);
		
		PushArrayArray(g_hAppartmentDoorAccess, nNew[0]);
	}
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	decl String:sQuery[2048];
	
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_building_appartment_access` (`playerID` int(10) NOT NULL, `appartmentID` int(10) NOT NULL, `accessID` int(10) NOT NULL, PRIMARY KEY (`playerID`,`appartmentID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	ResetAppartmentDoorAccess();
	LoadAppartmentDoorAccess();
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	//PlayerID is now cached
}

public RP_OnDoorLoaded(doorID,buildingType)
{
	if(buildingType == BD_AppartmentHouse)
	{
		decl appID;
		appID = RP_GetDoorAppartmentID(doorID);
		if(RP_GetDoorType(doorID) == Door_AppartmentHouse_Entrance || (RP_GetDoorType(doorID) == Door_AppartmentHouse_Appartment && !RP_IsAppartmenSold(appID)))
			RP_UnlockDoor(doorID);
	}
}

public RP_OnDoorUse(doorID,buildingType,client,Float:distance)
{
	if(buildingType == BD_AppartmentHouse)
	{
		if(0 >= client || client > MaxClients + 1)
			return;
		
		if(!IsClientInGame(client))
			return;
		
		if(!IsPlayerAlive(client))
			return;
			
		decl appID, access, doorType;
		appID = RP_GetDoorAppartmentID(doorID);
		access = GetPlayerAppartmentAccess(client, appID);
		doorType = RP_GetDoorType(doorID);
		
		if((access >= 0 && appID >= 0) || doorType == Door_AppartmentHouse_Entrance)
		{
			if (RP_IsValidLinearDoorEnt(RP_GetDoorEntity(doorID)) && distance <= APPARTMENT_DOOR_INTERACTION_RANGE)
				RP_ToggleOpenDoor(doorID);
		}

		if(GetGameTime() >= (g_fLastNote[client] + MAX_NOTE_INFO_TIME) && doorType != Door_AppartmentHouse_Entrance)
		{
			g_fLastNote[client] = GetGameTime();
			PrintToChat(client, "Use /info for door information");
		}
	}
}

public RP_OnDoubleShiftButton(client, entity)
{
	if(RP_IsValidDoorEnt(entity))
	{
		new appID = RP_GetDoorAppartmentID(RP_GetDoorIDbyEntity(entity));
		new access = GetPlayerAppartmentAccess(client, appID);
		
		if(access >= 0 && appID >= 0)
		{
			RP_ToggleLockDoorByEntity(entity);
		}
	}
}

Menu_AppartmentDoorMain(client)
{
	new access = RP_GetPlayerAppartmentAccess(client, g_iAppartmentSelection[client]);
	
	new Handle:menu = CreateMenu(MenuHandler_AppartmentDoorMain);
	SetMenuTitle(menu, "Appartment Door Menu");
	
	AddMenuItemEx(menu, false, "0", "Home Info");
	
	// Buy it! 
	if(!RP_IsAppartmenSold(g_iAppartmentSelection[client]))
	{
		AddMenuItemEx(menu, false, "1", "Buy Menu / Price: %d$", RP_GetAppartmentPrice(g_iAppartmentSelection[client]));
	}
	
	// You're welcome
	else if(access != -1)
	{		
		// Keeper
		if(access <= 1)
			AddMenuItemEx(menu, false, "2", "Manage Appartment Access");
		
		// Owner 
		if(access == 0)
			AddMenuItemEx(menu, false, "3", "Sell Home");
	}
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_AppartmentDoorMain(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new selection = StringToInt(info);
			
			if(selection == 0)
			{
				Menu_AppartmentInfoPanel(client);
			}
			else if(selection == 1)
			{
				Menu_BuyAppartmentDoor(client);
			}
			else if(selection == 2)
			{
				Menu_ManageAppartmentAccess(client);
			}
			else if(selection == 3)
			{
				Menu_SellHomeMenu(client);
			}
			return;
		}
	}
}

Menu_AppartmentInfoPanel(client)
{	
	new Handle:panel = CreateMenuEx(_,MenuHandler_AppartmentInfoPanel,MENU_ACTIONS_ALL);
	SetMenuPagination(panel,4);
	new String:text[511];
	new String:temp[511];
	
	new String:appnamebuffer[64];
	new String:buildingnamebuffer[64];
	new String:ownernamebuffer[64];
	
	RP_GetAppartmentRoomnumber(g_iAppartmentSelection[client], appnamebuffer, sizeof(appnamebuffer));
	RP_GetBuildingName(RP_GetAppartmentbuildingID(g_iAppartmentSelection[client]), buildingnamebuffer, sizeof(buildingnamebuffer));
	new ownerID = RP_GetAppartmentPlayerID(g_iAppartmentSelection[client]);
	if(ownerID > 0)
		RP_GetNameByPlayerID(ownerID, ownernamebuffer, sizeof(ownernamebuffer));
	
	Format(temp,sizeof(temp),"[%d] %s \n",g_iAppartmentSelection[client], appnamebuffer);
	StrCat(text,255,temp);
	AddMenuItem(panel,"",text);
	text="";
	
	Format(temp,sizeof(temp),"Building: [%d] %s \n",RP_GetAppartmentbuildingID(g_iAppartmentSelection[client]),buildingnamebuffer);
	StrCat(text,255,temp);
	AddMenuItem(panel,"",text);
	text="";
	
	Format(temp,sizeof(temp),"Doors: %d / Max_Inhibitants: %d \n",RP_GetAppartmentDoors(g_iAppartmentSelection[client]), RP_GetAppartmentMaxInh(g_iAppartmentSelection[client]));
	StrCat(text,255,temp);
	AddMenuItem(panel,"",text);
	text="";
	
	Format(temp,sizeof(temp),"Owner: [%d] %s \n",ownerID,ownernamebuffer);
	StrCat(text,255,temp);
	AddMenuItem(panel,"",text);
	text="";

	Format(temp,sizeof(temp),"Price: %d$ \n",RP_GetAppartmentPrice(g_iAppartmentSelection[client]));
	StrCat(text,255,temp);
	AddMenuItem(panel,"",text);
	text="";

	Format(temp,sizeof(temp),"Back to Appartment Menu \n");
	StrCat(text,255,temp);
	AddMenuItem(panel,"999",text);
	text="";

	DisplayMenu(panel,client,MENU_TIME_FOREVER);
}

public MenuHandler_AppartmentInfoPanel(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new selection = StringToInt(info);
			
			if(selection == 999)
			{
				Menu_AppartmentDoorMain(client);
			}
			return;
		}
	}
	
	if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
}

Menu_SellHomeMenu(client)
{
	new Handle:menu = CreateMenu(MenuHandler_SellHomeMenu);
	SetMenuTitle(menu, "Sell Home");
	
	AddMenuItemEx(menu, true, "", "!!!!!!!!!!!!!!!!!!!!!!!!!!!");
	AddMenuItemEx(menu, true, "", "Are you sure?");
	AddMenuItemEx(menu, true, "", "!!!!!!!!!!!!!!!!!!!!!!!!!!!");
	AddMenuItemEx(menu, false, "1", "YES");
	AddMenuItemEx(menu, false, "0", "NO");
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_SellHomeMenu(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new selection = StringToInt(info);
			
			if(selection == 0)
			{
				Menu_AppartmentDoorMain(client);
			}
			else if(selection == 1)
			{
				RP_SellAppartment(g_iAppartmentSelection[client]);
				ResetAppartmentPlayerAccesses(g_iAppartmentSelection[client]);
				RP_AddMoneyBank(client, RP_GetAppartmentPrice(g_iAppartmentSelection[client]));
				PrintToChat(client, "Sold home");
			}
			return;
		}
	}
}

Menu_BuyAppartmentDoor(client)
{
	new appPrice = RP_GetAppartmentPrice(g_iAppartmentSelection[client]);
	new String:appnamebuffer[64];
	
	RP_GetAppartmentRoomnumber(g_iAppartmentSelection[client], appnamebuffer, sizeof(appnamebuffer));
	
	new Handle:menu = CreateMenu(MenuHandler_BuyAppartmentDoor);
	SetMenuTitle(menu, "Appartment Buy Menu");
	
	if(!RP_IsAppartmenSold(g_iAppartmentSelection[client]))
	{
		AddMenuItemEx(menu, true, "", "[%d] %s",g_iAppartmentSelection[client], appnamebuffer);
		AddMenuItemEx(menu, true, "", "Doors: %d / Max_Inhibitants: %d",RP_GetAppartmentDoors(g_iAppartmentSelection[client]), RP_GetAppartmentMaxInh(g_iAppartmentSelection[client]));
		AddMenuItemEx(menu, true, "", "Price: %d$",appPrice);
		if(RP_GetMoneyBank(client) >= appPrice)
			AddMenuItemEx(menu, false, "0", "Buy with Bankaccount");
		else
			AddMenuItemEx(menu, true, "", "Not enough Money on your Bankaccount");

		AddMenuItemEx(menu, false, "1", "Back to overview");
		SetMenuExitButton(menu, true);
		DisplayMenu(menu, client, MENU_TIME_FOREVER);
	}
	else	
	{
		PrintToChat(client, "Cannot buy this due it's already sold");
		Menu_AppartmentDoorMain(client);
	}
	
}

public MenuHandler_BuyAppartmentDoor(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new selection = StringToInt(info);
			
			if(selection == 0)
			{
				new appPrice = RP_GetAppartmentPrice(g_iAppartmentSelection[client]);
				
				if(RP_GetMoneyBank(client) >= appPrice)
				{
					RP_TakeMoneyBank(client, appPrice);
					RP_SetAppartmentPlayerID(g_iAppartmentSelection[client], RP_GetPlayerID(client));
					RP_SetAppartmentSold(g_iAppartmentSelection[client], true);
					RP_SetPlayerAppartmentAccess(client, g_iAppartmentSelection[client], 0);
					
					new doorID;
					for (new i = 0; i < RP_GetDoorCount(); i++)
					{
						doorID = RP_GetDoorIDByArrayIndex(i);
						if(RP_GetDoorAppartmentID(doorID) == g_iAppartmentSelection[client])
						{
							new String:buffer[128];
							FormatEx(buffer, sizeof(buffer), "%N's Appartment", client);
							RP_SetDoorName(doorID, buffer);
						}
					}
					
					
				}
				else
				{
					PrintToChat(client, "Not enough Money on your Bankaccount");
					Menu_AppartmentDoorMain(client);
				}
			}
			else if(selection == 1)
			{
				Menu_AppartmentDoorMain(client);
			}
			return;
		}
	}
}

Menu_ManageAppartmentAccess(client)
{
	new Handle:menu = CreateMenu(MenuHandler_ManageAppartmentAccess);
	SetMenuTitle(menu, "Manage Appartment Access");
	
	if(RP_GetPlayerAppartmentPlayerCount(g_iAppartmentSelection[client]) > 1)
		AddMenuItemEx(menu, false, "0", "Edit Player Access");
	AddMenuItemEx(menu, false, "1", "Grand Player Access");
	AddMenuItemEx(menu, false, "2", "Back to overview");
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_ManageAppartmentAccess(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new selection = StringToInt(info);
			
			if(selection == 0)
			{
				if(RP_GetPlayerAppartmentPlayerCount(g_iAppartmentSelection[client]) > 1)
				{
					Menu_SelectAppartmentInhibitant(client);
				}
				else
				{
					PrintToChat(client, "There are no other inhibitants");
					Menu_AppartmentDoorMain(client);
				}
			}
			else if(selection == 1)
			{
				Menu_SelectPlayer(client);
			}
			else if(selection == 2)
			{
				Menu_AppartmentDoorMain(client);
			}
			return;
		}
	}
}

Menu_SelectAppartmentInhibitant(client)
{
	new Handle:menu = CreateMenu(MenuHandler_SelectAppartmentInhibitant);
	SetMenuTitle(menu, "Select Appartment Inhibitant");
	
	for (new i = 0; i < GetArraySize(g_hAppartmentDoorAccess); i++)
	{
		new nNew[RP_AppartmentAccess];
		GetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]);
		
		if (nNew[AppartmentAccess_AppartementID] == g_iAppartmentSelection[client] && nNew[AppartmentAccess_PlayerID] != RP_GetPlayerID(client))
		{
			new String:sPlayerID[64];
			new String:sPlayerName[64];
			IntToString(nNew[AppartmentAccess_PlayerID], sPlayerID, sizeof(sPlayerID));
			GetClientName(RP_GetClient(nNew[AppartmentAccess_PlayerID]), sPlayerName, sizeof(sPlayerName));
			AddMenuItemEx(menu, false, sPlayerID, "[%d] %s", nNew[AppartmentAccess_PlayerID],sPlayerName);
		}
	}
	AddMenuItemEx(menu, false, "back", "Back to overview");
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_SelectAppartmentInhibitant(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{			
			if(StrEqual(info,"back",false))
			{
				Menu_AppartmentDoorMain(client);
			}
			else
			{
				new selection = StringToInt(info);
				g_iPlayerIDSelection[client] = selection;
				Menu_SelectAccessType(client);
			}
			return;
		}
	}
}

Menu_SelectPlayer(client)
{	
	new Handle:menu = CreateMenu(MenuHandler_SelectPlayer);
	SetMenuTitle(menu, "[RP] Choose a Player");
	new String:buffer[8];
	new String:namebuffer[64];

	for (new i = 1; i < MAXPLAYERS; i++)
	{		
		if(!IsClientInGame(i))
			continue;
		
		if(IsFakeClient(i))
			continue;
		
		if(i != client)
		{
			GetClientName(i, namebuffer, sizeof(namebuffer));
		
			IntToString(i,buffer,sizeof(buffer));
			AddMenuItemEx(menu, false, buffer, "[%d] %s",RP_GetPlayerID(i),namebuffer);
		}
	}
	AddMenuItemEx(menu, false, "999", "Back to overview");

	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_SelectPlayer(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new selection = StringToInt(info);
			
			if(selection == 999)
			{
				Menu_AppartmentDoorMain(client);
			}
			else
			{
				g_iPlayerIDSelection[client] = selection;
				Menu_SelectAccessType(client);
			}
			return;
		}
	}
}

Menu_SelectAccessType(client)
{
	new String:sPlayerName[64];
	GetClientName(RP_GetClient(g_iPlayerIDSelection[client]), sPlayerName, sizeof(sPlayerName));
	
	new Handle:menu = CreateMenu(MenuHandler_SelectAccessType);
	SetMenuTitle(menu, "Select Accesstype for %s", sPlayerName);
	
	for (new i = 1; i < MAXACCESSTYPES; i++)
	{
		new String:sIndex[8];
		IntToString(i, sIndex, sizeof(sIndex));
		AddMenuItemEx(menu, false, sIndex, "%s", AccesTypeName[i]);
	}
	AddMenuItemEx(menu, false, "back", "Back to overview");
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_SelectAccessType(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{			
			if(StrEqual(info,"back",false))
			{
				Menu_AppartmentDoorMain(client);
			}
			else
			{
				new selection = StringToInt(info);
				new selectedClient = RP_GetClient(g_iPlayerIDSelection[client]);
				if(0 >= selectedClient || selectedClient > MaxClients + 1)
				{
					RP_SetPlayerAppartmentAccess(selectedClient, g_iAppartmentSelection[client], selection);
					new String:sClientName[64];
					GetClientName(selectedClient, sClientName, sizeof(sClientName));
					PrintToChat(client, "Set Accesslevel: %s to [%d] %s", AccesTypeName[selection], g_iPlayerIDSelection[client], sClientName);
					Menu_AppartmentDoorMain(client);
				}
			}
			return;
		}
	}
}

ResetAppartmentPlayerAccesses(appartmentID)
{
	for (new i = 0; i < GetArraySize(g_hAppartmentDoorAccess); i++)
	{
		new nNew[RP_AppartmentAccess];
		GetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]);
		
		if (nNew[AppartmentAccess_AppartementID] == appartmentID)
		{
			RemoveFromArray(g_hAppartmentDoorAccess, i);
			i--;
		}
	}
	
	decl String:sQuery[2048];
	
	Format(sQuery, sizeof(sQuery), "DELETE FROM `rp_building_appartment_access` WHERE appartmentID = %d;", appartmentID);
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_DeletePlayerAppartmentAccessTableCallback, sQuery, _, DBPrio_High);
}

public SQL_DeletePlayerAppartmentAccessTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_DeletePlayerAppartmentAccessTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

GetPlayerAppartmentPlayerCount(appartmentID)
{
	new counter = 0;
	for (new i = 0; i < GetArraySize(g_hAppartmentDoorAccess); i++)
	{
		new nNew[RP_AppartmentAccess];
		GetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]);
		
		if (nNew[AppartmentAccess_AppartementID] == appartmentID)
		{
			counter++;
		}
	}
	return counter;
}

GetPlayerAppartmentAccess(client, appartmentID)
{
	if(0 >= client || client > MaxClients + 1)
		return -1;
	
	if(!IsClientInGame(client))
		return -1;
	
	if(!IsPlayerAlive(client))
		return -1;
	
	new playerID = RP_GetPlayerID(client);
	
	for (new i = 0; i < GetArraySize(g_hAppartmentDoorAccess); i++)
	{
		new nNew[RP_AppartmentAccess];
		GetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]);
		
		if (nNew[AppartmentAccess_PlayerID] == playerID && nNew[AppartmentAccess_AppartementID] == appartmentID)
		{
			return nNew[AppartmentAccess_AccessID];
		}
	}
	
	return -1;

}

SetPlayerAppartmentAccess(client, appartmentID, new_accessID)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	new playerID = RP_GetPlayerID(client);
	new bool:insert = true;
	
	for (new i = 0; i < GetArraySize(g_hAppartmentDoorAccess); i++)
	{
		new nNew[RP_AppartmentAccess];
		GetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]);
		
		if (nNew[AppartmentAccess_PlayerID] == playerID && nNew[AppartmentAccess_AppartementID] == appartmentID)
		{
			nNew[AppartmentAccess_AccessID] = new_accessID;
			SetArrayArray(g_hAppartmentDoorAccess, i, nNew[0]); 
			insert = false;
		}
	}
	
	if(insert)
	{
		decl String:sQuery[2048];
	
		Format(sQuery, sizeof(sQuery), "INSERT INTO `rp_building_appartment_access` ( playerID, appartmentID , accessID) VALUES (%d, %d, %d);",playerID,appartmentID,new_accessID);
		LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
		SQL_TQuery(g_hSQL, SQL_InsertPlayerAppartmentAccessTableCallback, sQuery, _, DBPrio_High);
	}
}

public SQL_InsertPlayerAppartmentAccessTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_InsertPlayerAppartmentAccessTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	ResetAppartmentDoorAccess();
	LoadAppartmentDoorAccess();
}


public Native_GetPlayerAppartmentPlayerCount(Handle:plugin, numParams)
{
	return GetPlayerAppartmentPlayerCount(GetNativeCell(1));
}

public Native_GetPlayerAppartmentAccess(Handle:plugin, numParams)
{
	return GetPlayerAppartmentAccess(GetNativeCell(1),GetNativeCell(2));
}

public Native_SetPlayerAppartmentAccess(Handle:plugin, numParams)
{
	SetPlayerAppartmentAccess(GetNativeCell(1),GetNativeCell(2),GetNativeCell(3));
}