#include "roleplay/roleplay"

#define PL_NAME		"RP_BUILDINGS"
#define MAXBUILDINGTYPES 7

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

//ADT Arrays
new nBuildingsTeamplate[RP_Buildings];
new Handle:g_hBuildings = INVALID_HANDLE;

/* Type Names */
new String:BuildingTypeName[MAXBUILDINGTYPES][32] =  { "Default", "PoliceDepartment", "Bank", "HomeHouse", "Gangbase", "AppartmentHouse","Shop"};

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_GetBuildingCount", Native_GetMaxBuildingCount);
	CreateNative("RP_GetBuildingIDByArrayIndex", Native_GetBuildingIDByArrayIndex);
	
	CreateNative("RP_GetBuildingTypeName", Native_GetBuildingTypeName);
	CreateNative("RP_GetMaxBuildingType", Native_GetMaxBuildingType);
	
	CreateNative("RP_GetBuildingName", Native_GetBuildingName);
	CreateNative("RP_GetBuildingOwnerID", Native_GetBuildingOwnerID);
	CreateNative("RP_GetBuildingOwnerName", Native_GetBuildingOwnerName);
	CreateNative("RP_GetBuildingType", Native_GetBuildingType);
	
	CreateNative("RP_SetBuildingName", Native_SetBuildingName);
	CreateNative("RP_SetBuildingOwnerID", Native_SetBuildingOwnerID);
	CreateNative("RP_SetBuildingOwnerName", Native_SetBuildingOwnerName);
	CreateNative("RP_SetBuildingType", Native_SetBuildingType);
	
	CreateNative("RP_CreateBuilding", Native_CreateBuilding);
	
	return APLRes_Success;
}

public OnPluginStart()
{
	// Init ADT arrays
	g_hBuildings = CreateArray(sizeof(nBuildingsTeamplate));
	nBuildingsTeamplate[Building_ID] = 0;
	
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	decl String:sQuery[2048];
	
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_buildings` (`bID` int(10) AUTO_INCREMENT NOT NULL, `bName` varchar(64) NOT NULL,`bOwnerID` BOOL NOT NULL DEFAULT '0',  `bOwnerName` varchar(64) NOT NULL, `bType` int(10) NOT NULL, PRIMARY KEY (`bID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	ResetBuildings();
	LoadBuildings();
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

CreateBuilding(String:Name[],bType)
{
	decl String:sQuery[2048];
	
	Format(sQuery, sizeof(sQuery), "INSERT INTO `rp_buildings` (bName, bOwnerID, bOwnerName,bType) VALUES ('%s', %d, '%s', %d)",Name,-1,"World",bType);
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);	
	SQL_TQuery(g_hSQL, SQL_InsertTableCallback, sQuery, _, DBPrio_High);
}

public SQL_InsertTableCallback(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_InsertTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	ResetBuildings();
	LoadBuildings();
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	//PlayerID is now cached
}

ResetBuildings()
{
	ClearArray(g_hBuildings);
	g_hBuildings = CreateArray(sizeof(nBuildingsTeamplate));
}

LoadBuildings()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `bID`,`bName`,`bOwnerID`,`bOwnerName`,`bType` FROM `rp_buildings`;");
	SQL_TQuery(g_hSQL, SQL_LoadBuildingsCallback, sQuery, _, DBPrio_Low);
}

public SQL_LoadBuildingsCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadBuildingsCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_Buildings];
		
		nNew[Building_ID] = SQL_FetchInt(hndl, 0);
		SQL_FetchString(hndl, 1, nNew[Building_Name], 64);
		nNew[Building_OwnerID] = SQL_FetchInt(hndl, 2);
		SQL_FetchString(hndl,3, nNew[Building_OwnerName], 64);
		nNew[Building_Type] = SQL_FetchInt(hndl, 4);
		
		PushArrayArray(g_hBuildings, nNew[0]);
	}
}

SaveBuildings(arrayID)
{
	new nNew[RP_Buildings];
	GetArrayArray(g_hBuildings, arrayID, nNew[0]);
	
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "UPDATE `rp_buildings` SET bName = '%s', bOwnerID = %d ,bOwnerName = '%s', bType = %d WHERE bID = %d;",nNew[Building_Name],nNew[Building_OwnerID],nNew[Building_OwnerName],nNew[Building_Type],nNew[Building_ID]);
	SQL_TQuery(g_hSQL, SQL_SaveBuildingsCallback, sQuery, _, DBPrio_Low);
}

public SQL_SaveBuildingsCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_SaveBuildingsCallback) %s", PL_PRE_N, PL_NAME, error);
	}

}

public Native_GetMaxBuildingCount(Handle:plugin, numParams)
{
	return GetArraySize(g_hBuildings);
}

public Native_GetBuildingIDByArrayIndex(Handle:plugin, numParams)
{
	new arrayIndex = GetNativeCell(1);
	if(arrayIndex >= 0 || GetArraySize(g_hBuildings) > arrayIndex )
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, arrayIndex, nNew[0]);
		return nNew[Building_ID];
	}
	
	return -1;
}

GetBuildingOwnerID(buildingID)
{
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
			return nNew[Building_OwnerID];
	}
	
	return -1;
}

GetBuildingType(buildingID)
{
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
			return nNew[Building_Type];
	}
	
	return -1;
}

SetBuildingName(buildingID,String:new_Name[])
{
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
		{
			strcopy(nNew[Building_Name],64,new_Name);
			SetArrayArray(g_hBuildings, i, nNew[0]);
			SaveBuildings(i);
			break;
		}
	}
}
SetBuildingOwnerID(buildingID,new_OwnerID)
{
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
		{
			nNew[Building_OwnerID] = new_OwnerID;
			SetArrayArray(g_hBuildings, i, nNew[0]);
			SaveBuildings(i);
			break;
		}
	}
}

SetBuildingOwnerName(buildingID,String:new_Name[])
{
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
		{
			strcopy(nNew[Building_OwnerName] ,64,new_Name);
			SetArrayArray(g_hBuildings, i, nNew[0]);
			SaveBuildings(i);
			break;
		}
	}
}

SetBuildingType(buildingID,new_type)
{
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
		{
			nNew[Building_Type] = new_type;
			SetArrayArray(g_hBuildings, i, nNew[0]);
			SaveBuildings(i);
			break;
		}
	}
}

public Native_GetBuildingName(Handle:plugin, numParams)
{
	new nlen = GetNativeCell(3);
	new buildingID = GetNativeCell(1);
	
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
		{	
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Building_Name]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
		}	
	}
	
	return false;
}

public Native_GetBuildingOwnerID(Handle:plugin, numParams)
{
	return GetBuildingOwnerID(GetNativeCell(1));
}

public Native_GetBuildingOwnerName(Handle:plugin, numParams)
{
	new nlen = GetNativeCell(3);
	new buildingID = GetNativeCell(1);
	
	for (new i = 0; i < GetArraySize(g_hBuildings); i++)
	{
		new nNew[RP_Buildings];
		GetArrayArray(g_hBuildings, i, nNew[0]);
		
		if (nNew[Building_ID] == buildingID)
		{	
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Building_OwnerName]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
		}	
	}
	
	return false;
}

public Native_GetBuildingType(Handle:plugin, numParams)
{
	return GetBuildingType(GetNativeCell(1));
}

public Native_SetBuildingName(Handle:plugin, numParams)
{
	new String:buffer[64];
	
	GetNativeString(2,buffer,64);
	
	SetBuildingName(GetNativeCell(1),buffer);
}

public Native_SetBuildingOwnerID(Handle:plugin, numParams)
{
	SetBuildingOwnerID(GetNativeCell(1),GetNativeCell(2));
}

public Native_SetBuildingOwnerName(Handle:plugin, numParams)
{
	new String:buffer[64];
	
	GetNativeString(2,buffer,64);
	SetBuildingOwnerName(GetNativeCell(1),buffer);
}

public Native_SetBuildingType(Handle:plugin, numParams)
{
	SetBuildingType(GetNativeCell(1),GetNativeCell(2));
}

public Native_CreateBuilding(Handle:plugin, numParams)
{
	new String:buffer[64];
	
	GetNativeString(1,buffer,64);
	new btype = GetNativeCell(2);
	CreateBuilding(buffer,btype);
}

public Native_GetBuildingTypeName(Handle:plugin, numParams)
{
	new index = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	decl String:buffer[nlen];
	FormatEx(buffer, nlen, "%s", BuildingTypeName[index]);
	SetNativeString(2, buffer, nlen, true)
}

public Native_GetMaxBuildingType(Handle:plugin, numParams)
{
	return MAXBUILDINGTYPES;
}