#include "roleplay/roleplay"

#define PL_NAME		"RP_CLASH"

#define INTERVAL 1.0
#define COMBAT_TIMEOUT 45

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

new g_bHitgroup[MAXPLAYERS+1];

/* Forwards */
new Handle:g_OnPlayerKilled;
new Handle:g_OnPlayerHurt;
new Handle:g_OnPlayerCombatStart;
new Handle:g_OnPlayerCombatEnd;

/* Weapon Features */
new g_iWeaponDamageMin[MAXPLAYERS+1][5];
new g_iWeaponDamageMax[MAXPLAYERS+1][5];
new Float:g_fWeaponCritChance[MAXPLAYERS+1][5];
new Float:g_fWeaponBloodChance[MAXPLAYERS+1][5];
new Float:g_fWeaponIgniteChance[MAXPLAYERS+1][5];

/* TODO: More Weapon Features */
new g_iWeaponClipSize[MAXPLAYERS+1][5];
new bool:g_bWeaponBeathbeam[MAXPLAYERS+1][5];
new bool:g_bWeaponNVG[MAXPLAYERS+1][5];
new bool:g_bWeaponFlashlight[MAXPLAYERS+1][5];
new g_iWeaponScopeFOV[MAXPLAYERS+1][5];
new g_iWeaponLifeSteal[MAXPLAYERS+1][5];
new g_iWeaponMoneySteal[MAXPLAYERS+1][5];
new Float:g_fWeaponKnockoutChance[MAXPLAYERS+1][5];
new Float:g_fWeaponSlowChance[MAXPLAYERS+1][5];
new Float:g_fWeaponStunChance[MAXPLAYERS+1][5];

/* Cuff Features */
new bool:g_bIsCuffed[MAXPLAYERS+1];

/* Combat Timer */
new g_iCombatTimeout[MAXPLAYERS+1];
new bool:g_bCombat[MAXPLAYERS+1];

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	g_OnPlayerKilled = CreateGlobalForward("RP_OnPlayerKilled", ET_Event, Param_Cell, Param_Cell);
	g_OnPlayerHurt = CreateGlobalForward("RP_OnPlayerHurt", ET_Event, Param_Cell, Param_Cell);
	g_OnPlayerCombatStart = CreateGlobalForward("OnPlayerCombatStart", ET_Event, Param_Cell);
	g_OnPlayerCombatEnd = CreateGlobalForward("OnPlayerCombatEnd", ET_Event, Param_Cell);
	
	CreateNative("RP_IsPlayerInCombat", Native_IsPlayerInCombat);
	CreateNative("RP_CuffClient", Native_ClientCuff);
	CreateNative("RP_UncuffClient", Native_ClientUncuff);
	CreateNative("RP_IsClientCuffed", Native_IsClientCuffed);
	CreateNative("RP_ClientToggleCuff", Native_ClientToggleCuff);
	
	return APLRes_Success;
}

public OnPluginStart()
{
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
		
		/* Load players which are already connected */
		for(new i = 1; i <= MaxClients; i++)
		{
			if(!IsClientInGame(i))
				continue;
			
			OnClientPutInServer(i);
		}
	}
	
	HookEvent("item_equip", Event_ItemEquip);
	HookEvent("player_death", Event_Player_Death, EventHookMode_Pre);
	HookEvent("player_spawn", Event_PlayerSpawn);
	
	CreateTimer(INTERVAL, Timer_CombatTimeout, _, TIMER_REPEAT);
}

public OnClientPutInServer(client)
{
	g_bHitgroup[client] = 0;
	SDKHook(client, SDKHook_OnTakeDamage, Hook_OnTakeDamage);
	SDKHook(client, SDKHook_TraceAttack, Hook_OnTraceAttack);
	SDKHook(client, SDKHook_WeaponCanUse, OnWeaponCanUse);
}

public OnClientDisconnect(client)
{
	g_bHitgroup[client] = 0;
	SDKUnhook(client, SDKHook_OnTakeDamage, Hook_OnTakeDamage);
	SDKUnhook(client, SDKHook_TraceAttack, Hook_OnTraceAttack);
	SDKUnhook(client, SDKHook_WeaponCanUse, OnWeaponCanUse);
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_clash` (`playerID` int(10) NOT NULL, `cuffed` BOOL NOT NULL, PRIMARY KEY (`playerID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	if(create)
	{
		decl String:sQuery[2048];
		FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_clash` (playerID, cuffed) VALUES (%d, 0)", playerID);
		SQL_TQuery(g_hSQL, SQL_ClientCreateCallback, sQuery, GetClientUserId(client), DBPrio_High);
		
		SetPlayerDefaults(client);
	}
	else
	{
		LoadPlayer(client);
	}
}

public SQL_ClientCreateCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientCreateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

LoadPlayer(client)
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `cuffed` FROM `rp_clash` WHERE `playerID` = %d", RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientLoadCallback, sQuery, GetClientUserId(client), DBPrio_High);
}

public SQL_ClientLoadCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientConnectCallback) %s", PL_PRE_N, PL_NAME, error);
	}

	new client = GetClientOfUserId(userid);
	if(!client || !IsClientInGame(client))
		return;
	
	if(SQL_GetRowCount(hndl))
	{
		SQL_FetchRow(hndl);
		g_bIsCuffed[client] = bool:SQL_FetchInt(hndl, 0);
		
		if(g_bIsCuffed[client]){RP_CuffClient(client);}
	}
}

SetPlayerDefaults(client)
{
	g_bIsCuffed[client] = false;
}

SavePlayer(client)
{
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_clash` SET cuffed = %d WHERE playerID = %d", g_bIsCuffed[client], RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientUpdateCallback, sQuery, _);
}

public SQL_ClientUpdateCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientUpdateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

/* Hooks */

public Event_PlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	g_bHitgroup[client] = 0;
}

public Action:OnWeaponCanUse(Client, weapon)
{
	if(!IsClientInGame(Client))
		return Plugin_Continue;
	
	if(RP_IsClientCuffed(Client))
		return Plugin_Handled;
	
	/*if(RP_IsKnockedOut(client))
		return Plugin_Handled;*/
	
	/*if(RP_CanUseWeapon(client, weapon))
		return Plugin_Handled;*/
	
	return Plugin_Continue;
}

public Action:Event_ItemEquip(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));

	new type = GetEventInt(event, "weptype");
	
	new slot = -1;
	if(type == 0)
		slot = CS_SLOT_KNIFE;
	if(2 <= type <= 6)
		slot = CS_SLOT_PRIMARY;
	if(type == 1)
		slot = CS_SLOT_SECONDARY;
	if(type == 8)
		slot = CS_SLOT_GRENADE;
	if(slot == -1)
		slot = 0;
	
	ResetWeaponStats(client, slot);
	
	new itemID = RP_GetWeaponSlotItem(client, slot);
	
	if(itemID > 0)
	{
		decl String:sFeatures[128], String:sSettings[128], String:sName[64], String:sClassname[64];
		
		RP_GetItemFeatures(itemID, sFeatures, 128);
		RP_GetItemSettings(itemID, sSettings, 128);
		RP_GetItemClassname(itemID, sClassname, 128);
		RP_GetItemName(itemID, sName, 64);
		
		decl String:sFeature[MAX_ITEM_SETTINGS][128], String:sSetting[MAX_ITEM_SETTINGS][128];
		
		ExplodeString(sFeatures, ":", sFeature, sizeof(sFeature), sizeof(sFeatures));
		ExplodeString(sSettings, ":", sSetting, sizeof(sSetting), sizeof(sSettings));
		
		for (new i = 0; i < MAX_ITEM_SETTINGS; i++)
		{
			if(StrEqual(sFeature[i], "dmg_min"))
			{
				g_iWeaponDamageMin[client][slot] = StringToInt(sSetting[i]);
			}
			else if(StrEqual(sFeature[i], "dmg_max"))
			{
				g_iWeaponDamageMax[client][slot] = StringToInt(sSetting[i]);
			}
			else if(StrEqual(sFeature[i], "crit"))
			{
				g_fWeaponCritChance[client][slot] = StringToFloat(sSetting[i]);
			}
			else if(StrEqual(sFeature[i], "blood"))
			{
				g_fWeaponBloodChance[client][slot] = StringToFloat(sSetting[i]);
			}
			else if(StrEqual(sFeature[i], "ignite"))
			{
				g_fWeaponIgniteChance[client][slot] = StringToFloat(sSetting[i]);
			}
			else if(StrEqual(sFeature[i], "flashlight"))
			{
				g_bWeaponFlashlight[client][slot] = bool:StringToInt(sSetting[i]);
			}
			else if(StrEqual(sFeature[i], "nvg"))
			{
				g_bWeaponFlashlight[client][slot] = bool:StringToInt(sSetting[i]);
			}
		}
	}
	
	/* Flashlight */
	new m_fEffects = GetEntProp(client, Prop_Send, "m_fEffects");
	if(g_bWeaponFlashlight[client][slot])
		m_fEffects |= 4;
	else m_fEffects &= 4;
	SetEntProp(client, Prop_Send, "m_fEffects", m_fEffects); 
	
	/* NVG */
	if(g_bWeaponNVG[client][slot])
		SetEntProp(client, Prop_Send, "m_bNightVisionOn", 1);
	else SetEntProp(client, Prop_Send, "m_bNightVisionOn", 0);
}

bool:IsClientCuffed(client)
{
	return g_bIsCuffed[client];
}

ClientCuff(client)
{
	if(Client_IsValid(client))
	{	
		if(!RP_IsCop(client))
		{
			SetEntitySpeed(client,0.35);
			SetSpeed(client,90.0);
			SetEntityRenderMode(client, RENDER_GLOW);
			SetEntityRenderColor(client,0,0,255,255);
			g_bIsCuffed[client] = true;
			SavePlayer(client);
		}
	}	
}

ClientUncuff(client)
{
	if(Client_IsValid(client))
	{	
		if(!RP_IsCop(client) && IsClientCuffed(client))
		{
			SetEntitySpeed(client,1.0);
			SetSpeed(client,400.0);
			SetEntityRenderMode(client, RENDER_NORMAL);
			SetEntityRenderColor(client,255,255,255,255);
			g_bIsCuffed[client] = false;
			SavePlayer(client);
		}
	}
}

ClientToggleCuff(client)
{
	if(IsClientCuffed(client))
	{
		ClientUncuff(client);
	}
	else
	{
		ClientCuff(client);
	}
}


ResetWeaponStats(client, slot)
{
	g_iWeaponDamageMin[client][slot] = 0;
	g_iWeaponDamageMax[client][slot] = 0;
	
	g_fWeaponCritChance[client][slot] = 0.0;
	g_fWeaponBloodChance[client][slot] = 0.0;
	g_fWeaponIgniteChance[client][slot] = 0.0;
	
	g_bWeaponFlashlight[client][slot] = false;
	g_bWeaponNVG[client][slot] = false;
}

public Action:Hook_OnTraceAttack(victim, &attacker, &inflictor, &Float:damage, &damagetype, &ammotype, hitbox, hitgroup)
{
	if(victim && victim <= MaxClients)
		g_bHitgroup[victim] = hitgroup;
	
	return Plugin_Continue;
}

public Action:Hook_OnTakeDamage(victim, &attacker, &inflictor, &Float:damage, &damagetype, &weapon, Float:damageForce[3], Float:damagePosition[3])
{
	if(!Client_IsValid(attacker))
		return Plugin_Continue;
	
	if(victim == attacker)
		return Plugin_Continue;
	
	new Float:initial_damage = damage;
	
	decl String:sGrenade[32], String:sWeapon[32];
	
	GetEdictClassname(inflictor, sGrenade, sizeof(sGrenade));
	GetClientWeapon(attacker, sWeapon, sizeof(sWeapon));
	PrintToChat(attacker, "Test");
	BalanceDamage(victim, attacker, sWeapon, sGrenade, damage, g_bHitgroup[victim]);
	
	if(StrEqual(sWeapon, "weapon_taser"))
	{
		new todo_knockout;
		RP_CuffClient(victim);
	}
	if(StrEqual(sWeapon, "weapon_knife") && GetClientTeam(attacker) == CS_TEAM_CT)
	{
		RP_ClientToggleCuff(victim);
	}

	
	/* Forward: RP_OnPlayerHurt */
	Call_StartForward(g_OnPlayerHurt);
	Call_PushCell(victim);
	Call_PushCell(attacker);
	Call_Finish();
	
	CombatAction(attacker);
	CombatAction(victim);
	
	//No damage done
	if(damage <= 0.0)
		return Plugin_Handled;
	
	//Non lethal damage done
	if(damage < 1.0)
		damage = 1.0;
	
	//Damage changed
	if(initial_damage != damage)
		return Plugin_Changed;
	
	return Plugin_Continue;
}

public Action:Event_Player_Death(Handle:event, const String:name[], bool:dontBroadcast)
{
	new victim = GetClientOfUserId(GetEventInt(event, "userid"));
	new attacker = GetClientOfUserId(GetEventInt(event, "attacker"));
	
	if(!Client_IsValid(attacker))
		return Plugin_Continue;
	
	if(victim == attacker)
		return Plugin_Continue;
	
	/* Forward: RP_OnPlayerKilled */
	Call_StartForward(g_OnPlayerKilled);
	Call_PushCell(victim);
	Call_PushCell(attacker);
	Call_Finish();
	
	CombatAction(attacker);
	
	// Fake headshot event if needed
	if (g_bHitgroup[victim] == 1)
	{
		decl String:weapon[64];
		GetEventString(event, "weapon", weapon, sizeof(weapon));
		
		new Handle:new_event = CreateEvent("player_death");
		if (new_event == INVALID_HANDLE)
		{
			return Plugin_Continue;
		}
		
		SetEventInt(new_event, "userid", GetClientUserId(victim));
		SetEventInt(new_event, "attacker", GetClientUserId(attacker));
		SetEventString(new_event, "weapon", weapon);
		SetEventBool(new_event, "headshot", true);
		FireEvent(new_event);
		return Plugin_Changed;
	}

	return Plugin_Continue;
}

BalanceDamage(victim, attacker, String:sWeapon[], String:sGrenade[], &Float:damage, hitgroup)
{
	//Check godmode features first
	/*if(RP_GetLevel(victim) < 1 || RP_GetLevel(attacker) < 1)
	{
		damage = 0.0;
		return;
	}

	if(RP_IsTouchingGodmodeZone(victim) || RP_IsTouchingGodmodeZone(attacker))
	{
		damage = 0.0;
		return;
	}*/
	/*
	if(g_bGodmode[victim])
	{
		damage = 0.0;
		return;
	}*/
	
	PrintToChat(attacker, "1: %f", damage);
	
	new slot;
	//Get slot
	if(IsWeaponGrenade(sGrenade))
	{
		slot = WeaponGetSlot(sWeapon);
	}
	
	//Basic damage
	//damage = float(g_iWeaponDamageMin[attacker][slot]);
	
	//Bonus damage
	//damage += GetRandomFloat(0.0, float(g_iWeaponDamageMax[attacker][slot])-damage);
	
	//Level Bonus Damage (1% per level)
	damage *= 1.0 + (float(RP_GetLevel(attacker)) / 100.0);
	PrintToChat(attacker, "3: %f", damage);
	
	//Level Reduced Damage
	//damage -= damage * (100.0/((float(RP_GetLevel(attacker))^0.5-float(RP_GetLevel(attacker))/50.0)*5.0));
	
	//Crit chance
	if(GetRandomInt(0, 100) < g_fWeaponCritChance[attacker][slot])
		damage *= 2.0;
	
	//Ignite chance
	if(GetRandomInt(0, 100) < g_fWeaponIgniteChance[attacker][slot])
	{
		new todo_ignite_burn;
	}
	
	//Blood chance
	if(GetRandomInt(0, 100) < g_fWeaponBloodChance[attacker][slot])
	{
		new todo_blood_hp_loss;
	}
	
	//...more
	
	PrintToChat(attacker, "5: %f", damage);
	
	//Check Hitgroups
	new todo_add_hitgroup_items; //Armor
	
	//Generic
	if(hitgroup == 0)
	{
		damage *= 1.0;
	}
	//Head
	else if(hitgroup == 1)
	{
		damage *= 0.5;
	}
	//Chest
	else if(hitgroup == 2)
	{
		damage *= 0.5;
	}
	//Stomach
	else if(hitgroup == 3)
	{
		damage *= 0.6;
	}
	//LeftArm
	else if(hitgroup == 4)
	{
		damage *= 0.6;
	}
	//RightArm
	else if(hitgroup == 5)
	{
		damage *= 0.6;
	}
	//LeftLeg
	else if(hitgroup == 6)
	{
		damage *= 0.4;
	}
	//RightLeg
	else if(hitgroup == 7)
	{
		damage *= 0.4;
	}
	//Gear
	else if(hitgroup == 8)
	{
		new todo_damage_gear;
		damage *= 0.2;
	}
	
	PrintToChat(attacker, "6: %f", damage);
	
	//Check Teams
	if(GetClientTeam(attacker) == CS_TEAM_CT && GetClientTeam(victim) == CS_TEAM_CT)
	{
		damage *= 0.5;
	}
	
	//Check Taser && Knife for CT (Cuff / Uncuff)
	if(GetClientTeam(attacker) == CS_TEAM_CT && GetClientTeam(victim) == CS_TEAM_T && (StrEqual(sWeapon, "weapon_taser") || StrEqual(sWeapon, "weapon_knife")))
	{
		damage = 0.0;
	}
	PrintToChat(attacker, "7: %f", damage);
}

/* Combat Status */

CombatReset(client)
{
	g_bCombat[client] = false;
	
	g_iCombatTimeout[client] = GetTime();
	
	if(IsClientInGame(client))
	{
		/* Forward: RP_OnPlayerCombatEnd */
		Call_StartForward(g_OnPlayerCombatEnd);
		Call_PushCell(client);
		Call_Finish();
	}
}

CombatAction(client)
{
	if(!g_bCombat[client])
	{
		g_bCombat[client] = true;
		
		/* Forward: RP_OnPlayerCombatStart */
		Call_StartForward(g_OnPlayerCombatStart);
		Call_PushCell(client);
		Call_Finish();
	}
	
	/* Reset combat timeout */
	g_iCombatTimeout[client] = GetTime() + COMBAT_TIMEOUT;
}

public Action:Timer_CombatTimeout(Handle:timer, any:data)
{
	new iTime = GetTime();
	for (new client = 1; client < MAXPLAYERS ; client++)
	{
		if(!g_bCombat[client])
			continue;
		
		if(g_iCombatTimeout[client] < iTime)
			CombatReset(client);
	}
	
	return Plugin_Continue;
}

/* Natives */

public Native_IsPlayerInCombat(Handle:plugin, numParams)
{
	return g_bCombat[GetNativeCell(1)];
}

public Native_IsClientCuffed(Handle:plugin, numParams)
{
	return IsClientCuffed(GetNativeCell(1));
}

public Native_ClientCuff(Handle:plugin, numParams)
{
	ClientCuff(GetNativeCell(1));
}

public Native_ClientUncuff(Handle:plugin, numParams)
{
	ClientUncuff(GetNativeCell(1));
}

public Native_ClientToggleCuff(Handle:plugin, numParams)
{
	ClientToggleCuff(GetNativeCell(1));
}
