#include "roleplay/roleplay"

#define PL_NAME		"RP_ITEMS"

#define ITEM_ERROR_NOT_FOUND -1
#define ITEM_ERROR_NOT_ENOUGH -2
#define ITEM_ERROR_REQ_ITEM -3
#define ITEM_ERROR_REQ_LEVEL -4

#define MAXITEMTYPES 17

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

/* Templates */
new nItemTeamplate[RP_Item];
new Handle:g_hItems = INVALID_HANDLE;

new nItemGroupTeamplate[RP_Item];
new Handle:g_hItemGroups = INVALID_HANDLE;

new nInventoryTeamplate[RP_Inventory];
new Handle:g_hInventory[MAXPLAYERS+1] = {INVALID_HANDLE, ...};


/* For Menu Usage */
new bool:UsedItemType[MAXITEMTYPES] = {false, ...};
new String:ItemTypeName[MAXITEMTYPES][32] =  { "Other", "Valuable", "Furniture", "SpecialWeapon", "Weapon", "Ammo", "Food", "Drink", "Medic", "Tool", "Drug", "Document", "Badge", "Tattoo", "Skin", "Mask", "Home" };

/* Entity 2 itemID cache */
new g_iItemID[2048];

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_GetItemCount", Native_GetItemCount);
	CreateNative("RP_GetItemID", Native_GetItemID);
	
	CreateNative("RP_GetItemTypeUsed", Native_GetItemTypeUsed);
	CreateNative("RP_GetItemTypeName", Native_GetItemTypeName);
	CreateNative("RP_GetMaxItemType", Native_GetMaxItemType);
	
	CreateNative("RP_GetItemType", Native_GetItemType);
	CreateNative("RP_GetItemSubType", Native_GetItemSubType);
	CreateNative("RP_GetItemWorthBuy", Native_GetItemWorthBuy);
	CreateNative("RP_GetItemWorthSell", Native_GetItemWorthSell);
	CreateNative("RP_GetItemDurability", Native_GetItemDurability);
	
	CreateNative("RP_GetItemReqLevel", Native_GetItemReqLevel);
	CreateNative("RP_GetItemReqItem", Native_GetItemReqItem);
	CreateNative("RP_GetItemReqItemDurability", Native_GetItemReqItemDurability);
	
	CreateNative("RP_IsItemTradable", Native_IsItemTradable);
	CreateNative("RP_IsItemDropable", Native_IsItemDropable);
	CreateNative("RP_IsItemUseable", Native_IsItemUseable);
	CreateNative("RP_IsItemDeleteable", Native_IsItemDeleteable);
	CreateNative("RP_IsItemVipOnly", Native_IsItemVipOnly);
	
	CreateNative("RP_GetItemWeight ", Native_GetItemWeight);
	
	CreateNative("RP_GetItemTeam", Native_GetItemTeam);
	
	CreateNative("RP_GetItemCrimePickup", Native_GetItemCrimePickup);
	CreateNative("RP_GetItemCrimeBuy", Native_GetItemCrimeBuy);
	CreateNative("RP_GetItemCrimeSell", Native_GetItemCrimeSell);
	CreateNative("RP_GetItemCrimeUse", Native_GetItemCrimeUse);
	
	CreateNative("RP_GetItemName", Native_GetItemName);
	CreateNative("RP_GetItemClassname", Native_GetItemClassname);
	CreateNative("RP_GetItemModel", Native_GetItemModel);
	CreateNative("RP_GetItemFeatures", Native_GetItemFeatures);
	CreateNative("RP_GetItemSettings", Native_GetItemSettings);
	CreateNative("RP_GetItemDescription", Native_GetItemDescription);
	
	/* Player Inventory */
	CreateNative("RP_GetPlayerInventoryItemCount", Native_GetPlayerInventoryItemCount);
	CreateNative("RP_GetPlayerInventoryItemTypeCount", Native_GetPlayerInventoryItemTypeCount);
	CreateNative("RP_GetPlayerInventoryItemSubTypeCount", Native_GetPlayerInventoryItemSubTypeCount);
	CreateNative("RP_GetPlayerInventoryItem", Native_GetPlayerInventoryItem);

	CreateNative("RP_AddPlayerInventoryItem", Native_AddPlayerInventoryItem);
	CreateNative("RP_UsedPlayerInventoryItem", Native_UsedPlayerInventoryItem);
	CreateNative("RP_SellPlayerItem", Native_SellPlayerItem);
	CreateNative("RP_TakePlayerItem", Native_TakePlayerItem);
	CreateNative("RP_RemovePlayerInventoryItem", Native_RemovePlayerInventoryItem);
	CreateNative("RP_RemovePlayerInventoryItemType", Native_RemovePlayerInventoryItemType);
	CreateNative("RP_RemovePlayerInventoryItemCrime", Native_RemovePlayerInventoryItemCrime);
	CreateNative("RP_HasPlayerInventoryItem", Native_HasPlayerInventoryItem);
	
	CreateNative("RP_GetEntityItem", Native_GetEntityItem);
	CreateNative("RP_SetEntityItem", Native_SetEntityItem);
	
	new todo_custom_inventorys;
	
	return APLRes_Success;
}

public OnPluginStart()
{
	/* Init ADT arrays */
	g_hItems = CreateArray(sizeof(nItemTeamplate));
	nItemTeamplate[Item_ID] = 0;
	
	for(new i = 1; i <= MaxClients; i++)
		g_hInventory[i] = CreateArray(sizeof(nInventoryTeamplate));
	nInventoryTeamplate[Inventory_ID] = 0;
	
	/* Perform action if plugin is late loaded */
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	/* Create Tables */
	
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_items` (`itemID` int(10) NOT NULL AUTO_INCREMENT, `itemType` int(10) NOT NULL, `itemSubType` int(10) NOT NULL, `worthBuy` int(11) NOT NULL, `worthSell` int(11) NOT NULL, `durability` int(11) NOT NULL, `req_level` int(11) NOT NULL, `req_item` int(11) NOT NULL, `req_item_durability` int(11) NOT NULL, `tradeable` int(1) NOT NULL, `dropable` int(1) NOT NULL, `useable` int(1) NOT NULL, `deleteable` int(1) NOT NULL, `vip` int(1) NOT NULL, `weight` int(11) NOT NULL, `team` int(11) NOT NULL, `crime_pickup` int(11) NOT NULL, `crime_buy` int(11) NOT NULL, `crime_sell` int(11) NOT NULL, `crime_use` int(11) NOT NULL, `name` varchar(64) NOT NULL, `classname` varchar(64) NOT NULL, `model` varchar(128) NOT NULL, `features` varchar(%d) NOT NULL, `settings` varchar(%d) NOT NULL, `description` varchar(512) NOT NULL, PRIMARY KEY (`itemID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;", MAX_FEATURE_LENGTH, MAX_FEATURE_LENGTH);
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_player_inventory` (`id` int(10) NOT NULL AUTO_INCREMENT, `playerID` int(10) NOT NULL, `itemID` int(10) NOT NULL, `durability` int(11) NOT NULL, `date_created` int(11) NOT NULL, PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	ResetItems();
	LoadItems();
	
	/* Load players which are already connected */
	for(new i = 1; i <= MaxClients; i++)
	{
		if(!IsClientInGame(i))
			continue;
		if(IsFakeClient(i))
			continue;
		
		ResetInventory(i);
		LoadInventory(i);
	}
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

/* Items */

ResetItems()
{
	ClearArray(g_hItems);
	g_hItems = CreateArray(sizeof(nItemTeamplate));
}

LoadItems()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `itemID`,`itemType`,`itemSubType`,`worthBuy`,`worthSell`,`durability`,`req_level`,`req_item`,`req_item_durability`,`tradeable`,`dropable`,`useable`,`deleteable`,`vip`,`weight`,`team`,`crime_pickup`,`crime_buy`,`crime_sell`,`crime_use`,`name`,`classname`,`model`,`features`,`settings`,`description` FROM `rp_items`;");
	SQL_TQuery(g_hSQL, SQL_LoadItemsCallback, sQuery, _, DBPrio_Low);
}

public SQL_LoadItemsCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadItemsCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	/* Fetch all items */
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_Item];
		
		nNew[Item_ID] = SQL_FetchInt(hndl, 0);
		nNew[Item_Type] = _:SQL_FetchInt(hndl, 1);
		nNew[Item_SubType] = _:SQL_FetchInt(hndl, 2);
		nNew[Item_WorthBuy] = SQL_FetchInt(hndl, 3);
		nNew[Item_WorthSell] = SQL_FetchInt(hndl, 4);
		nNew[Item_Durability] = SQL_FetchInt(hndl, 5);
		nNew[Item_ReqLevel] = SQL_FetchInt(hndl, 6);
		nNew[Item_ReqItem] = SQL_FetchInt(hndl, 7);
		nNew[Item_ReqItemDurability] = SQL_FetchInt(hndl, 8);
		nNew[Item_Tradable] = bool:SQL_FetchInt(hndl, 9);
		nNew[Item_Dropable] = bool:SQL_FetchInt(hndl, 10);
		nNew[Item_Useable] = bool:SQL_FetchInt(hndl, 11);
		nNew[Item_Deleteable] = bool:SQL_FetchInt(hndl, 12);
		nNew[Item_VipOnly] = bool:SQL_FetchInt(hndl, 13);
		nNew[Item_Weight] = SQL_FetchInt(hndl, 14);
		nNew[Item_Team] = SQL_FetchInt(hndl, 15);
		nNew[Item_CrimePickup] = SQL_FetchInt(hndl, 16);
		nNew[Item_CrimeBuy] = SQL_FetchInt(hndl, 17);
		nNew[Item_CrimeSell] = SQL_FetchInt(hndl, 18);
		nNew[Item_CrimeUse] = SQL_FetchInt(hndl, 19);
		SQL_FetchString(hndl, 20, nNew[Item_Name], 64);
		SQL_FetchString(hndl, 21, nNew[Item_Classname], 64);
		SQL_FetchString(hndl, 22, nNew[Item_Model], 128);
		SQL_FetchString(hndl, 23, nNew[Item_Features], MAX_FEATURE_LENGTH);
		SQL_FetchString(hndl, 24, nNew[Item_Settings], MAX_FEATURE_LENGTH);
		SQL_FetchString(hndl, 25, nNew[Item_Description], 512);
		
		UsedItemType[SQL_FetchInt(hndl, 1)] = true;
		
		PushArrayArray(g_hItems, nNew[0]);
	}
}

GetItemType(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Type];
		}
	}
	
	return -1;
}

GetItemSubType(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_SubType];
		}
	}
	
	return -1;
}

GetItemWorthBuy(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_WorthBuy];
		}
	}
	
	return -1;
}

GetItemWorthSell(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_WorthSell];
		}
	}
	
	return -1;
}

GetItemDurability(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Durability];
		}
	}
	
	return -1;
}

GetItemReqLevel(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_ReqLevel];
		}
	}
	
	return -1;
}

GetItemReqItem(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_ReqItem];
		}
	}
	
	return -1;
}

GetItemReqItemDurability(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_ReqItemDurability];
		}
	}
	
	return -1;
}

bool:IsItemTradable(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Tradable];
		}
	}
	
	return false;
}

bool:IsItemDropable(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Dropable];
		}
	}
	
	return false;
}

bool:IsItemUseable(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Useable];
		}
	}
	
	return false;
}

bool:IsItemDeleteable(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Deleteable];
		}
	}
	
	return false;
}

bool:IsItemVipOnly(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_VipOnly];
		}
	}
	
	return false;
}

GetItemWeight(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Weight];
		}
	}
	
	return -1;
}

GetItemTeam(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_Team];
		}
	}
	
	return -1;
}

GetItemCrimePickup(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_CrimePickup];
		}
	}
	
	return -1;
}

GetItemCrimeBuy(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_CrimeBuy];
		}
	}
	
	return -1;
}

GetItemCrimeSell(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_CrimeSell];
		}
	}
	
	return -1;
}

GetItemCrimeUse(itemID)
{
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			return nNew[Item_CrimeUse];
		}
	}
	
	return -1;
}

/* Inventory */

public RP_OnPlayerReady(client, playerID, bool:create)
{
	ResetInventory(client);
	LoadInventory(client);
}

public OnClientDisconnect(client)
{
	ResetInventory(client);
}

ResetInventory(client)
{
	if(g_hInventory[client] != INVALID_HANDLE) 
		ClearArray(g_hInventory[client]);
	g_hInventory[client] = CreateArray(sizeof(nInventoryTeamplate));
}

LoadInventory(client)
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `itemID`,`durability`,`date_created` FROM `rp_player_inventory` WHERE `playerID` = %d", RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientLoadInventoryCallback, sQuery, GetClientUserId(client), DBPrio_Low);
}

public SQL_ClientLoadInventoryCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientLoadInventoryCallback) %s", PL_PRE_N, PL_NAME, error);
	}

	new client = GetClientOfUserId(userid);
	if(!client || !IsClientInGame(client))
		return;
	
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_Inventory];
		
		nNew[Inventory_ID] = SQL_FetchInt(hndl, 0);
		nNew[Inventory_Durability] = SQL_FetchInt(hndl, 1);
		nNew[Inventory_DateCreated] = SQL_FetchInt(hndl, 2);
		PushArrayArray(g_hInventory[client], nNew[0]);
	}
	
	new todo_sort_inventory;
}

ClientAddItem(client, itemID, durability)
{
	if(RP_GetItemReqItem(itemID) != -1)
	{
		if(ClientHasItem(client, RP_GetItemReqItem(itemID)) == 0)
			return ITEM_ERROR_REQ_ITEM;
	}
	if(RP_GetItemReqLevel(itemID) > RP_GetLevel(client))
		return ITEM_ERROR_REQ_LEVEL;
	
	new todo_check_vip;
	
	for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (nNew[Inventory_ID] == itemID)
		{
			nNew[Inventory_Durability] += durability;
			
			SetArrayArray(g_hInventory[client], i, nNew[0]);
			
			decl String:sQuery[2048];
			FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_player_inventory` SET durability = %d WHERE playerID = %d AND itemID = %d", nNew[Inventory_Durability], RP_GetPlayerID(client), itemID);
			SQL_TQuery(g_hSQL, SQL_ClientUpdateInventoryCallback, sQuery, _);
			return nNew[Inventory_Durability];
		}
	}
	
	new nNew[RP_Inventory];
	nNew[Inventory_ID] = itemID;
	nNew[Inventory_Durability] = durability;
	nNew[Inventory_DateCreated] = GetTime();
	PushArrayArray(g_hInventory[client], nNew[0]);
	
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_player_inventory` (playerID, itemID, durability, date_created) VALUES (%d, %d, %d, %d)", RP_GetPlayerID(client), itemID, durability, nNew[Inventory_DateCreated]);
	SQL_TQuery(g_hSQL, SQL_ClientInsertInventoryCallback, sQuery, _);
	return durability;
}

ClientUsedItem(client, itemID, durability)
{
	if(RP_GetItemReqItem(itemID) != -1)
	{
		if(ClientHasItem(client, RP_GetItemReqItem(itemID)) == 0)
			return ITEM_ERROR_REQ_ITEM;
	}
	
	if(RP_GetItemReqLevel(itemID) > RP_GetLevel(client))
		return ITEM_ERROR_REQ_LEVEL;
	
	new todo_check_vip;

	for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (nNew[Inventory_ID] == itemID)
		{
			if(nNew[Inventory_Durability] < durability)
				return ITEM_ERROR_NOT_ENOUGH;
			
			nNew[Inventory_Durability] -= durability;
			SetArrayArray(g_hInventory[client], i, nNew[0]);
			
			if(nNew[Inventory_Durability] > 0)
			{				
				decl String:sQuery[2048];
				FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_player_inventory` SET durability = %d WHERE playerID = %d AND itemID = %d", nNew[Inventory_Durability], RP_GetPlayerID(client), itemID);
				SQL_TQuery(g_hSQL, SQL_ClientUpdateInventoryCallback, sQuery, _);
				return nNew[Inventory_Durability];
			}
			else
			{
				ClientRemoveItem(client, itemID);
				return 0;
			}
		}
	}
	
	return ITEM_ERROR_NOT_FOUND;
}

ClientSellItem(client, itemID, durability, price)
{
	for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (nNew[Inventory_ID] == itemID)
		{
			if(nNew[Inventory_Durability] < durability)
				return;
			
			nNew[Inventory_Durability] -= durability;
			RP_AddMoney(client, price);
			SetArrayArray(g_hInventory[client], i, nNew[0]);
			
			if(nNew[Inventory_Durability] > 0)
			{				
				decl String:sQuery[2048];
				FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_player_inventory` SET durability = %d WHERE playerID = %d AND itemID = %d", nNew[Inventory_Durability], RP_GetPlayerID(client), itemID);
				SQL_TQuery(g_hSQL, SQL_ClientUpdateInventoryCallback, sQuery, _);
				return;
			}
			else
			{
				ClientRemoveItem(client, itemID);
				return;
			}
		}
	}
	
	return;
}

ClientTakeItem(client, itemID, durability)
{
	for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (nNew[Inventory_ID] == itemID)
		{	
			nNew[Inventory_Durability] -= durability;
			SetArrayArray(g_hInventory[client], i, nNew[0]);
			
			if(nNew[Inventory_Durability] > 0)
			{				
				decl String:sQuery[2048];
				FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_player_inventory` SET durability = %d WHERE playerID = %d AND itemID = %d", nNew[Inventory_Durability], RP_GetPlayerID(client), itemID);
				SQL_TQuery(g_hSQL, SQL_ClientUpdateInventoryCallback, sQuery, _);
				return;
			}
			else
			{
				ClientRemoveItem(client, itemID);
				return;
			}
		}
	}
	
	return;
}

ClientRemoveItem(client, itemID)
{
	new bool:found;
	
	for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (nNew[Inventory_ID] == itemID)
		{
			RemoveFromArray(g_hInventory[client], i);
			found = true;
			break;
		}
	}
	
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "DELETE FROM `rp_player_inventory` WHERE playerID = %d AND itemID = %d", RP_GetPlayerID(client), itemID);
	SQL_TQuery(g_hSQL, SQL_ClientDeleteInventoryCallback, sQuery, _);
	
	return found;
}

public ClientRemoveItemType(client, type)
{
	new iCount;
	
	for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (GetItemType(nNew[Inventory_ID]) == type)
		{
			RemoveFromArray(g_hInventory[client], i);
			
			decl String:sQuery[2048];
			FormatEx(sQuery, sizeof(sQuery), "DELETE FROM `rp_player_inventory` WHERE playerID = %d AND itemID = %d", RP_GetPlayerID(client), nNew[Inventory_ID]);
			SQL_TQuery(g_hSQL, SQL_ClientDeleteInventoryCallback, sQuery, _);
			
			iCount++;
		}
	}
	
	return iCount;
}

public ClientRemoveItemCrime(client, crime)
{
	new iCount;
	
	for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (GetItemCrimePickup(nNew[Inventory_ID]) > 0 || GetItemCrimeBuy(nNew[Inventory_ID]) > 0 || GetItemCrimeSell(nNew[Inventory_ID]) > 0 || GetItemCrimeUse(nNew[Inventory_ID]) > 0)
		{
			RemoveFromArray(g_hInventory[client], i);
			
			decl String:sQuery[2048];
			FormatEx(sQuery, sizeof(sQuery), "DELETE FROM `rp_player_inventory` WHERE playerID = %d AND itemID = %d", RP_GetPlayerID(client), nNew[Inventory_ID]);
			SQL_TQuery(g_hSQL, SQL_ClientDeleteInventoryCallback, sQuery, _);
			
			iCount++;
		}
	}
	
	return iCount;
}

ClientHasItem(client, itemID)
{
	if(IsClientInGame(client))
	{
		for (new i = 0; i < GetArraySize(g_hInventory[client]); i++)
		{
			new nNew[RP_Inventory];
			GetArrayArray(g_hInventory[client], i, nNew[0]);
		
			if (nNew[Inventory_ID] == itemID)
			{
				return nNew[Inventory_Durability];
			}
		}
	}
	
	return 0;
}

public SQL_ClientUpdateInventoryCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientUpdateInventoryCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public SQL_ClientInsertInventoryCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientInsertInventoryCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public SQL_ClientDeleteInventoryCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientDeleteInventoryCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

/* Entity Cache */

public OnEntityDestroyed(entity)
{
	if(entity < 2047 && entity >= 0)
		g_iItemID[entity] = 0;
}

/* Natives */

public Native_GetItemCount(Handle:plugin, numParams)
{
	return GetArraySize(g_hItems);
}

public Native_GetItemID(Handle:plugin, numParams)
{
	new nNew[RP_Item];
	GetArrayArray(g_hItems, GetNativeCell(1), nNew[0]);
	return nNew[Item_ID];
}

public Native_GetItemType(Handle:plugin, numParams)
{
	return GetItemType(GetNativeCell(1));
}

public Native_GetItemSubType(Handle:plugin, numParams)
{
	return GetItemSubType(GetNativeCell(1));
}

public Native_GetItemWorthBuy(Handle:plugin, numParams)
{
	return GetItemWorthBuy(GetNativeCell(1));
}

public Native_GetItemWorthSell(Handle:plugin, numParams)
{
	return GetItemWorthSell(GetNativeCell(1));
}

public Native_GetItemDurability(Handle:plugin, numParams)
{
	return GetItemDurability(GetNativeCell(1));
}

public Native_GetItemReqLevel(Handle:plugin, numParams)
{
	return GetItemReqLevel(GetNativeCell(1));
}

public Native_GetItemReqItem(Handle:plugin, numParams)
{
	return GetItemReqItem(GetNativeCell(1));
}

public Native_GetItemReqItemDurability(Handle:plugin, numParams)
{
	return GetItemReqItemDurability(GetNativeCell(1));
}

public Native_IsItemTradable(Handle:plugin, numParams)
{
	return IsItemTradable(GetNativeCell(1));
}

public Native_IsItemDropable(Handle:plugin, numParams)
{
	return IsItemDropable(GetNativeCell(1));
}

public Native_IsItemUseable(Handle:plugin, numParams)
{
	return IsItemUseable(GetNativeCell(1));
}

public Native_IsItemDeleteable(Handle:plugin, numParams)
{
	return IsItemDeleteable(GetNativeCell(1));
}

public Native_IsItemVipOnly(Handle:plugin, numParams)
{
	return IsItemVipOnly(GetNativeCell(1));
}

public Native_GetItemWeight(Handle:plugin, numParams)
{
	return GetItemWeight(GetNativeCell(1));
}

public Native_GetItemTeam(Handle:plugin, numParams)
{
	return GetItemTeam(GetNativeCell(1));
}

public Native_GetItemCrimePickup(Handle:plugin, numParams)
{
	return GetItemCrimePickup(GetNativeCell(1));
}

public Native_GetItemCrimeBuy(Handle:plugin, numParams)
{
	return GetItemCrimeBuy(GetNativeCell(1));
}

public Native_GetItemCrimeSell(Handle:plugin, numParams)
{
	return GetItemCrimeSell(GetNativeCell(1));
}

public Native_GetItemCrimeUse(Handle:plugin, numParams)
{
	return GetItemCrimeUse(GetNativeCell(1));
}

public Native_GetItemName(Handle:plugin, numParams)
{
	new itemID = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Item_Name]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
			else return false;
		}
	}
	
	return false;
}

public Native_GetItemClassname(Handle:plugin, numParams)
{
	new itemID = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Item_Classname]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
			else return false;
		}
	}
	
	return false;
}

public Native_GetItemModel(Handle:plugin, numParams)
{
	new itemID = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Item_Model]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
			else return false;
		}
	}
	
	return false;
}

public Native_GetItemFeatures(Handle:plugin, numParams)
{
	new itemID = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Item_Features]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
			else return false;
		}
	}
	
	return false;
}

public Native_GetItemSettings(Handle:plugin, numParams)
{
	new itemID = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Item_Settings]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
			else return false;
		}
	}
	
	return false;
}

public Native_GetItemDescription(Handle:plugin, numParams)
{
	new itemID = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Item];
		GetArrayArray(g_hItems, i, nNew[0]);
		
		if (nNew[Item_ID] == itemID)
		{
			decl String:buffer[nlen];
			FormatEx(buffer, nlen, "%s", nNew[Item_Description]);
			if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
				return true;
			else return false;
		}
	}
	
	return false;
}

public Native_GetPlayerInventoryItemCount(Handle:plugin, numParams)
{
	return GetArraySize(g_hInventory[GetNativeCell(1)]);
}

public Native_GetPlayerInventoryItemTypeCount(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new type = GetNativeCell(2);
	new iCount;
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (GetItemType(nNew[Inventory_ID]) == type)
			iCount += nNew[Inventory_Durability];
	}
	
	return iCount;
}

public Native_GetPlayerInventoryItemSubTypeCount(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new type = GetNativeCell(2);
	new subType = GetNativeCell(3);
	new iCount;
	
	for (new i = 0; i < GetArraySize(g_hItems); i++)
	{
		new nNew[RP_Inventory];
		GetArrayArray(g_hInventory[client], i, nNew[0]);
		
		if (GetItemType(nNew[Inventory_ID]) == type && GetItemSubType(nNew[Inventory_ID]) == subType)
			iCount += nNew[Inventory_Durability];
	}
	
	return iCount;
}

public Native_GetPlayerInventoryItem(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new index = GetNativeCell(2);
	
	if(index >= GetArraySize(g_hInventory[client]))
		return -1;
	
	new nNew[RP_Inventory];
	GetArrayArray(g_hInventory[client], index, nNew[0]);
	
	return nNew[Inventory_ID];
}

public Native_AddPlayerInventoryItem(Handle:plugin, numParams)
{
	return ClientAddItem(GetNativeCell(1), GetNativeCell(2), GetNativeCell(3));
}

public Native_UsedPlayerInventoryItem(Handle:plugin, numParams)
{
	return ClientUsedItem(GetNativeCell(1), GetNativeCell(2), GetNativeCell(3));
}

public Native_SellPlayerItem(Handle:plugin, numParams)
{
	ClientSellItem(GetNativeCell(1), GetNativeCell(2), GetNativeCell(3), GetNativeCell(4));
}

public Native_TakePlayerItem(Handle:plugin, numParams)
{
	ClientTakeItem(GetNativeCell(1), GetNativeCell(2), GetNativeCell(3));
}

public Native_RemovePlayerInventoryItem(Handle:plugin, numParams)
{
	return ClientRemoveItem(GetNativeCell(1), GetNativeCell(2));
}

public Native_RemovePlayerInventoryItemType(Handle:plugin, numParams)
{
	return ClientRemoveItemType(GetNativeCell(1), GetNativeCell(2));
}

public Native_RemovePlayerInventoryItemCrime(Handle:plugin, numParams)
{
	return ClientRemoveItemCrime(GetNativeCell(1), GetNativeCell(2));
}

public Native_HasPlayerInventoryItem(Handle:plugin, numParams)
{
	return ClientHasItem(GetNativeCell(1), GetNativeCell(2));
}

public Native_GetEntityItem(Handle:plugin, numParams)
{
	return g_iItemID[GetNativeCell(1)];
}

public Native_SetEntityItem(Handle:plugin, numParams)
{
	g_iItemID[GetNativeCell(1)] = GetNativeCell(2);
}

public Native_GetItemTypeUsed(Handle:plugin, numParams)
{
	return UsedItemType[GetNativeCell(1)];
}

public Native_GetItemTypeName(Handle:plugin, numParams)
{
	new index = GetNativeCell(1);
	new nlen = GetNativeCell(3);
	
	decl String:buffer[nlen];
	FormatEx(buffer, nlen, "%s", ItemTypeName[index]);
	SetNativeString(2, buffer, nlen, true)
}

public Native_GetMaxItemType(Handle:plugin, numParams)
{
	return MAXITEMTYPES;
}