#include "roleplay/roleplay"

#define PL_NAME		"RP_LEVEL"

#define INTERVAL 1.0

#define MAXLEVEL 100.0

#define MINEXP 120
#define EXPMULTI 1.1337

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

/* Forwards */
new Handle:g_OnPlayerLevelUp;

new g_iExp[MAXPLAYERS+1];
new g_iLevel[MAXPLAYERS+1];

new Handle:g_hReqEXP = INVALID_HANDLE;

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_GetExp", Native_GetExp);
	CreateNative("RP_AddExp", Native_AddExp);
	CreateNative("RP_SetExp", Native_SetExp);
	
	CreateNative("RP_GetLevel", Native_GetLevel);
	CreateNative("RP_AddLevel", Native_AddLevel);
	CreateNative("RP_SetLevel", Native_SetLevel);
	
	g_OnPlayerLevelUp = CreateGlobalForward("RP_OnPlayerLevelUp", ET_Event, Param_Cell, Param_Cell);
	
	return APLRes_Success;
}

public OnPluginStart()
{
	// Perform action if plugin is late loaded
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
	
	g_hReqEXP = CreateArray(1);
	PushArrayCell(g_hReqEXP, 0);
	
	for (new lvl = 1; lvl < MAXLEVEL ; lvl++)
	{
		new exp = GetArrayCell(g_hReqEXP, lvl-1);
		exp += RoundToFloor(EXPMULTI*lvl*MINEXP);
		PushArrayCell(g_hReqEXP, exp);
	}
	
	CreateTimer(INTERVAL, Timer_CheckExp, _, TIMER_REPEAT);
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_player_level` (`playerID` int(10) NOT NULL, `exp` int(11) NOT NULL, `level` int(11) NOT NULL, PRIMARY KEY (`playerID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	if(create)
	{
		decl String:sQuery[2048];
		FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_player_level` (playerID, exp, level) VALUES (%d, 0, 0)", RP_GetPlayerID(client));
		SQL_TQuery(g_hSQL, SQL_ClientCreateCallback, sQuery, GetClientUserId(client), DBPrio_Normal);
		
		SetPlayerDefaults(client);
	}
	else
	{
		LoadPlayer(client);
	}
}

public SQL_ClientCreateCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientCreateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

LoadPlayer(client)
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `exp`, `level` FROM `rp_player_level` WHERE `playerID` = %d", RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientLoadCallback, sQuery, GetClientUserId(client), DBPrio_Low);
}

public SQL_ClientLoadCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientConnectCallback) %s", PL_PRE_N, PL_NAME, error);
	}

	new client = GetClientOfUserId(userid);
	if(!client || !IsClientInGame(client))
		return;
	
	if(SQL_GetRowCount(hndl))
	{
		SQL_FetchRow(hndl);
		g_iExp[client] = SQL_FetchInt(hndl, 0);
		g_iLevel[client] = SQL_FetchInt(hndl, 1);
	}
}

SetPlayerDefaults(client)
{
	g_iExp[client] = 0;
	g_iLevel[client] = 0;
}

public Action:Timer_CheckExp(Handle:timer, any:data)
{
	if(g_hSQL == INVALID_HANDLE)
		return Plugin_Continue;
	
	for (new client = 1; client < MAXPLAYERS ; client++)
	{
		if (!IsClientInGame(client))
			continue;
		
		if (!IsPlayerAlive(client))
			continue;
		
		if (g_iLevel[client] >= MAXLEVEL)
			continue;
		
		if (g_iExp[client] > GetArrayCell(g_hReqEXP, g_iLevel[client]+1))
		{
			g_iLevel[client]++;
			
			Call_StartForward(g_OnPlayerLevelUp);
			Call_PushCell(client);
			Call_PushCell(g_iLevel[client]);
			Call_Finish();
		}
	}
	
	return Plugin_Continue;
}

/* Natives */

public Native_GetExp(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	
	return g_iExp[client];
}

public Native_AddExp(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new amount = GetNativeCell(2);
	
	g_iExp[client] += amount;
	
	return g_iExp[client];
}

public Native_SetExp(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new amount = GetNativeCell(2);
	
	g_iExp[client] = amount;
	
	return g_iExp[client];
}

public Native_GetLevel(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	
	return g_iLevel[client];
}

public Native_AddLevel(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new amount = GetNativeCell(2);
	
	g_iLevel[client] += amount;
	
	return g_iLevel[client];
}

public Native_SetLevel(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new amount = GetNativeCell(2);
	
	g_iLevel[client] = amount;
	
	return g_iLevel[client];
}