#include "roleplay/roleplay"

#define PL_NAME		"RP_MONEY"

#define TRANSFER_TAX 0.05
#define TRANSFER_TAX_MIN 5

#define CASH_TRANSFER_RANGE 128.0

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

new g_iMoney[MAXPLAYERS+1];
new g_iMoneyBank[MAXPLAYERS+1];

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	/* Register Natives */
	CreateNative("RP_GetMoney", Native_GetMoney);
	CreateNative("RP_SetMoney", Native_SetMoney);
	CreateNative("RP_AddMoney", Native_AddMoney);
	CreateNative("RP_TakeMoney", Native_TakeMoney);
	
	CreateNative("RP_GetMoneyBank", Native_GetMoneyBank);
	CreateNative("RP_SetMoneyBank", Native_SetMoneyBank);
	CreateNative("RP_AddMoneyBank", Native_AddMoneyBank);
	CreateNative("RP_TakeMoneyBank", Native_TakeMoneyBank);
	
	CreateNative("RP_TransferMoneyCash", Native_TransferMoneyCash);
	CreateNative("RP_TransferMoneyBank", Native_TransferMoneyBank);

	return APLRes_Success;
}

public OnPluginStart()
{
	RegConsoleCmd("sm_money", Command_Money);
	RegConsoleCmd("sm_givemoney", Command_GiveMoney);
	
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	/* Create Tables */
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_money` (`playerID` int(10) NOT NULL, `money` int(11) NOT NULL, `bank` int(11) NOT NULL, PRIMARY KEY (`playerID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_transaction` (`transactionID` int(10) NOT NULL AUTO_INCREMENT, `playerID` int(10) NOT NULL, `targetID` int(10) NOT NULL, `money` int(11) NOT NULL, `bank_transfer` int(1) NOT NULL, `date` int(11) NOT NULL, PRIMARY KEY (`transactionID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	/* Load players which are already connected */
	for(new i = 1; i <= MaxClients; i++)
	{
		if(!IsClientInGame(i))
			continue;
		if(IsFakeClient(i))
			continue;
		
		LoadPlayer(i)
	}
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public OnClientDisconnect(client)
{
	if(client > 0 && client < MAXPLAYERS + 1)
	{
		SavePlayer(client);
		g_iMoney[client] = 0;
		g_iMoneyBank[client] = 0;
	}
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	if(create)
	{
		decl String:sQuery[2048];
		FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_money` (playerID, money, bank) VALUES (%d, %d, %d)", RP_GetPlayerID(client), 0, 0);
		SQL_TQuery(g_hSQL, SQL_ClientCreateCallback, sQuery, GetClientUserId(client), DBPrio_Normal);
		
		SetPlayerDefaults(client);
	}
	else
	{
		LoadPlayer(client);
	}
}

public SQL_ClientCreateCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientCreateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

LoadPlayer(client)
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `money`,`bank` FROM `rp_money` WHERE `playerID` = %d", RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientLoadCallback, sQuery, GetClientUserId(client), DBPrio_Low);
}

public SQL_ClientLoadCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientConnectCallback) %s", PL_PRE_N, PL_NAME, error);
	}

	new client = GetClientOfUserId(userid);
	if(!client || !IsClientInGame(client))
		return;
	
	if(SQL_GetRowCount(hndl))
	{
		SQL_FetchRow(hndl);
		g_iMoney[client] = SQL_FetchInt(hndl, 0);
		g_iMoneyBank[client] = SQL_FetchInt(hndl, 1);
	}
}

SetPlayerDefaults(client)
{
	g_iMoney[client] = 0;
	g_iMoneyBank[client] = 0;
}

LogTransfer(playerID, targetID, money, bool:bank_transfer)
{
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_transaction` (playerID, targetID, money, bank_transfer, date) VALUES (%d, %d, %d, %d, %d)", playerID, targetID, money, bank_transfer, GetTime());
	SQL_TQuery(g_hSQL, SQL_TransferCallback, sQuery, _, DBPrio_Normal);
}

public SQL_TransferCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_TransferCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

SavePlayer(client)
{
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_money` SET money = %d,  bank = %d WHERE playerID = %d", g_iMoney[client], g_iMoneyBank[client], RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientUpdateCallback, sQuery, _);
}

PlayerIDAddMoney(playerID, amount)
{
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_money` SET bank = bank+%d WHERE playerID = %d", amount, playerID);
	SQL_TQuery(g_hSQL, SQL_ClientUpdateCallback, sQuery, _);
}

public SQL_ClientUpdateCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientUpdateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

/* Commands */

public Action:Command_Money(client, args)
{
	if(0 >= client > MaxClients)
		return Plugin_Handled;
	
	if(!IsClientInGame(client))
		return Plugin_Handled;
	
	if(!IsPlayerAlive(client))
		return Plugin_Handled;
	
	CPrintToChat(client, "%s Cash: $%d Bank: $%d", PL_PRE, g_iMoney[client], g_iMoneyBank[client]);
	return Plugin_Handled;
}

public Action:Command_GiveMoney(client, args)
{
	if(0 >= client > MaxClients)
		return Plugin_Handled;
	
	if(!IsClientInGame(client))
		return Plugin_Handled;
	
	if(!IsPlayerAlive(client))
		return Plugin_Handled;
	
	new target = GetClientAimTarget(client, true);
	
	if(!target || !IsClientInGame(target) || !IsPlayerAlive(target))
		CPrintToChat(client, "%s You have to look at the target player.", PL_PRE);
	
	if(!Entity_InRange(client, target, CASH_TRANSFER_RANGE))
	{
		CPrintToChat(client, "%s %N is not in range.", PL_PRE, target);
		return Plugin_Handled;
	}
	
	Menu_GiveMoney(client, target);
	return Plugin_Handled;
}

new g_iGiveMoneyTarget[MAXPLAYERS+1];

Menu_GiveMoney(client, target)
{
	g_iGiveMoneyTarget[client] = target;
	
	new Handle:menu = CreateMenu(MenuHandler_GiveMoney);
	SetMenuTitle(menu, "[Cash: $%d] Give cash to %N", g_iMoney[client], target);
	
	AddMenuItemEx(menu, !(g_iMoney[client] > 1), "1", "$1");
	AddMenuItemEx(menu, !(g_iMoney[client] > 10), "10", "$10");
	AddMenuItemEx(menu, !(g_iMoney[client] > 100), "100", "$100");
	AddMenuItemEx(menu, !(g_iMoney[client] > 1000), "1000", "$1000");
	AddMenuItemEx(menu, !(g_iMoney[client] > 10000), "10000", "$10000");
	
	decl String:sAll[32];
	IntToString(g_iMoney[client], sAll, sizeof(sAll));
	AddMenuItemEx(menu, !(g_iMoney[client] > 0), sAll, "$%d", g_iMoney[client])
	
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_GiveMoney(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client > MaxClients)
		return;
	
	new target = g_iGiveMoneyTarget[client];
	
	if(!target)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsClientInGame(target))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if(!IsPlayerAlive(target))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new amount = StringToInt(info);
			
			if(RP_TransferMoneyCash(client, target, amount))
			{
				CPrintToChat(client, "%s You have given $%d to %N.", PL_PRE, amount, target);
				CPrintToChat(target, "%s %N has given $%d to you.", PL_PRE, client, amount);
			}
			else CPrintToChat(client, "%s Transfer failed!", PL_PRE);
			
			Menu_GiveMoney(client, target);
		}
	}
}

/* Natives */

public Native_GetMoney(Handle:plugin, numParams)
{
	return g_iMoney[GetNativeCell(1)];
}

public Native_SetMoney(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	g_iMoney[client] = GetNativeCell(2);
	
	if(g_iMoney[client] < 0)
		g_iMoney[client] = 0;
	
	SavePlayer(client);
	return g_iMoney[client];
}

public Native_AddMoney(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	g_iMoney[client] += GetNativeCell(2);
	
	SavePlayer(client);
	return g_iMoney[client];
}

public Native_TakeMoney(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	g_iMoney[client] -= GetNativeCell(2);
	
	if(g_iMoney[client] < 0)
		g_iMoney[client] = 0;
	
	SavePlayer(client);
	return g_iMoney[client];
}

public Native_GetMoneyBank(Handle:plugin, numParams)
{
	return g_iMoneyBank[GetNativeCell(1)];
}

public Native_SetMoneyBank(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	g_iMoneyBank[client] = GetNativeCell(2);
	
	if(g_iMoneyBank[client] < 0)
		g_iMoneyBank[client] = 0;
	
	SavePlayer(client);
	return g_iMoneyBank[client];
}

public Native_AddMoneyBank(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	g_iMoneyBank[client] += GetNativeCell(2);
	
	SavePlayer(client);
	return g_iMoneyBank[client];
}

public Native_TakeMoneyBank(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	g_iMoneyBank[client] -= GetNativeCell(2);
	
	if(g_iMoneyBank[client] < 0)
		g_iMoneyBank[client] = 0;
	
	SavePlayer(client);
	return g_iMoneyBank[client];
}

public Native_TransferMoneyCash(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new target = GetNativeCell(2);
	new amount = GetNativeCell(3);
	
	if(!IsClientInGame(target))
		return false;
	
	if(IsFakeClient(target))
		return false;
	
	if(RP_GetPlayerID(client) <= 0)
		return false;
	
	if(RP_GetPlayerID(target) <= 0)
		return false;
	
	if(amount <= 0)
		return false;
	
	if(amount > g_iMoney[client])
		return false;
	
	g_iMoney[client] -= amount;
	g_iMoney[target] += amount;
	
	SavePlayer(client);
	SavePlayer(target);
	LogTransfer(RP_GetPlayerID(client), RP_GetPlayerID(target), amount, false);
	return true;
}

public Native_TransferMoneyBank(Handle:plugin, numParams)
{
	new client = GetNativeCell(1);
	new targetID = GetNativeCell(2);
	new amount = GetNativeCell(3);
	new playerID = RP_GetPlayerID(client);
	
	if(playerID <= 0)
		return false;
	
	if(targetID <= 0)
		return false;
	
	if(amount <= 0)
		return false;
	
	if(amount > g_iMoneyBank[client])
		return false;
	
	new tax = RoundToFloor(TRANSFER_TAX*amount)
	if(tax < TRANSFER_TAX_MIN)
		tax = TRANSFER_TAX_MIN;
	
	g_iMoneyBank[client] -= amount+tax;
	
	new target = RP_GetClient(targetID);
	
	if(target > 0)
	{
		g_iMoneyBank[target] += amount;
	}
	else PlayerIDAddMoney(playerID, amount);
	
	LogTransfer(playerID, targetID, amount, true);
	SavePlayer(client);
	SavePlayer(target);
	return true;
}