#include "roleplay/roleplay"

#define PL_NAME		"RP_VENDORS"

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

new nVendorTeamplate[RP_Vendor];
new Handle:g_hVendors = INVALID_HANDLE;


/* Forwards */
new Handle:g_OnBuyItem;
new Handle:g_OnSellItem;

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_AddVendorItem", Native_AddVendorItem);
	
	CreateNative("RP_GetVendorCount", Native_GetVendorCount);
	CreateNative("RP_GetVendorIDByIndex", Native_GetVendorIDByIndex);
	CreateNative("RP_GetVendorIndexByVendorID", Native_GetVendorIndexByVendorID);

	CreateNative("RP_GetVendorItemCount", Native_GetVendorItemCount);
	CreateNative("RP_GetVendorItemTypeCount", Native_GetVendorItemTypeCount);
	CreateNative("RP_GetVendorItemSubTypeCount", Native_GetVendorItemSubTypeCount);

	CreateNative("RP_GetVendorNpcID", Native_GetVendorNpcID);
	CreateNative("RP_GetVendorMultiplier", Native_GetVendorMultiplier);
	
	/* Register Forwards */
	g_OnBuyItem = CreateGlobalForward("RP_OnBuyItem", ET_Event, Param_Cell, Param_Cell);
	g_OnSellItem = CreateGlobalForward("RP_OnSellItem", ET_Event, Param_Cell, Param_Cell);

	return APLRes_Success;
}

public OnPluginStart()
{
	/* Init ADT arrays */
	g_hVendors = CreateArray(sizeof(nVendorTeamplate));
	nVendorTeamplate[Vendor_ID] = 0;
	
	/* Perform action if plugin is late loaded */
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
	
	RegAdminCmd("rp_vendor_mgr", Cmd_VendorMgr, ADMFLAG_ROOT);
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

SQLConnectionReady()
{
	/* Create Table */
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_vendors` (`vendorID` int(10) NOT NULL AUTO_INCREMENT, `npcID` int(10) NOT NULL, `itemID` int(10) NOT NULL, `multiplier` float NOT NULL, PRIMARY KEY (`vendorID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	ResetVendors();
	LoadVendors();
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

ResetVendors()
{
	ClearArray(g_hVendors);
	g_hVendors = CreateArray(sizeof(nVendorTeamplate));
}

LoadVendors()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `vendorID`,`npcID`,`itemID`,`multiplier` FROM `rp_vendors`;");
	SQL_TQuery(g_hSQL, SQL_LoadVendorsCallback, sQuery, _, DBPrio_Low);
}

public SQL_LoadVendorsCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadVendorsCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_Vendor];
		
		nNew[Vendor_ID] = SQL_FetchInt(hndl, 0);
		nNew[Vendor_NpcID] = SQL_FetchInt(hndl, 1);
		nNew[Vendor_ItemID] = SQL_FetchInt(hndl, 2);
		nNew[Vendor_Multiplier] = SQL_FetchFloat(hndl, 3);
		
		PushArrayArray(g_hVendors, nNew[0]);
	}
}

new g_iSelectedNpcID[MAXPLAYERS+1];
new g_iSelectedItemID[MAXPLAYERS+1];

public RP_OnPlayerNpcUse(client, npcID)
{
	if(RP_GetNpcType(npcID) != NT_Vendor)
		return;
	
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	g_iSelectedNpcID[client] = npcID;
	Menu_Trader(client);
}

Menu_Trader(client)
{
	new npcID = g_iSelectedNpcID[client];
	
	decl String:sName[64];
	RP_GetNpcName(npcID, sName, sizeof(sName));

	new Handle:menu = CreateMenu(MenuHandler_Trader);
	SetMenuTitle(menu, "Trader: %s", sName);
	
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		new itemID = nNew[Vendor_ItemID];
		
		if(!HasNpcItem(npcID, itemID))
			continue;
		
		decl String:sItemID[10];
		IntToString(itemID, sItemID, sizeof(sItemID));
		
		decl String:sItemName[64];
		RP_GetItemName(itemID, sItemName, sizeof(sItemName));
		
		if(!RP_IsItemTradable(itemID))
			continue;
		
		AddMenuItemEx(menu, false, sItemID, "%s [%d$]", sItemName, RP_GetItemWorthBuy(itemID));
	}
	
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_Trader(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!Client_IsValid(client))
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new itemID = StringToInt(info);
			g_iSelectedItemID[client] = itemID;
			Menu_ItemInfo(client);
			return;
		}
	}
}

Menu_ItemInfo(client)
{
	new itemID = g_iSelectedItemID[client];
	
	decl String:sItemName[64];
	RP_GetItemName(itemID, sItemName, sizeof(sItemName));
	
	decl String:sItemID[10];
	IntToString(itemID, sItemID, sizeof(sItemID));
	
	decl String:sItemDescription[64];
	RP_GetItemDescription(itemID, sItemDescription, sizeof(sItemDescription));
	
	new cash = RP_GetMoney(client);
	
	new Handle:menu = CreateMenu(MenuHandler_ItemInfo);
	SetMenuTitle(menu, "Item: %dx %s%s\nDescription: %s\nCash: $%d", RP_HasPlayerInventoryItem(client, itemID), RP_IsItemDropable(itemID) ? "(VIP) ":"", sItemName, sItemDescription, cash);
	
	new Float:fMultiplier = GetVendorMultiplier(g_iSelectedNpcID[client], itemID);
	new iBuy = RoundToFloor(float(RP_GetItemWorthBuy(itemID))*fMultiplier);
	new iSell = RP_GetItemWorthSell(itemID);
	
	new durability = RP_GetItemDurability(itemID);
	
	AddMenuItemEx(menu, false, "trader", "Back to trader");
	if(iBuy > 0) 
		AddMenuItemEx(menu, false , "buy", "Buy %d for $%d", durability, iBuy);
	else
		AddMenuItemEx(menu, true , "buy", "Not buyable");
		
	if(iSell > 0) 
		AddMenuItemEx(menu, false, "sell", "Sell %d for $%d", durability, iSell);
	else
		AddMenuItemEx(menu, true , "sell", "Not sellable");
	
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_ItemInfo(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client || client > MaxClients + 1)
		return;
	
	if(!IsClientInGame(client))
		return;
	
	if(!IsPlayerAlive(client))
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new itemID = g_iSelectedItemID[client];
			
			decl String:sItemName[64];
			RP_GetItemName(itemID, sItemName, sizeof(sItemName));
			
			decl String:sItemID[10];
			IntToString(itemID, sItemID, sizeof(sItemID));
			
			decl String:sItemDescription[64];
			RP_GetItemDescription(itemID, sItemDescription, sizeof(sItemDescription));
			
			new cash = RP_GetMoney(client);
			
			new Float:fMultiplier = GetVendorMultiplier(g_iSelectedNpcID[client], itemID);
			new iBuy = RoundToFloor(float(RP_GetItemWorthBuy(itemID))*fMultiplier);
			new iSell = RP_GetItemWorthSell(itemID);
			
			new durability = RP_GetItemDurability(itemID);
			
			if(StrEqual(info, "trader"))
			{
				Menu_Trader(client);
			}
			
			new bool:invalid;
			
			// Check req. item
			new req_item = RP_GetItemReqItem(itemID);
			if(req_item > 0 && RP_HasPlayerInventoryItem(client, req_item) >= RP_GetItemReqItemDurability(itemID))
			{
				decl String:sReqItemName[64];
				RP_GetItemName(req_item, sReqItemName, sizeof(sReqItemName));
				CPrintToChat(client, "%s You can't do this without a %s.", PL_PRE, sReqItemName);
				invalid = true;
			}
			
			new req_level = RP_GetItemReqLevel(itemID);
			// Check req. level
			if(req_level > RP_GetLevel(client))
			{
				CPrintToChat(client, "%s You need at least level %d to use a %s.", PL_PRE, req_level, sItemName);
				invalid = true;
			}
			
			// Vip only
			new todo_check_vip;
			/*
			if(RP_IsItemVipOnly(itemID) && !RP_ClientIsVIP(client))
				invalid = true;
			*/
			
			if(StrEqual(info, "buy"))
			{
				/* Forward: RP_OnBuyItem */
				Call_StartForward(g_OnBuyItem);
				Call_PushCell(client);
				Call_PushCell(itemID);
				Call_Finish();
				
				if(iBuy > cash)
				{
					CPrintToChat(client, "%s You don't have enough cash to buy this now.", PL_PRE);
					invalid = true;
				}
				
				if(!invalid)
				{
					RP_AddPlayerInventoryItem(client, itemID, durability);
					RP_TakeMoney(client, iBuy);
					CPrintToChat(client, "%s You have bought %dx %s.", PL_PRE, durability, sItemName);
				}
				
				Menu_ItemInfo(client);
			}
			else if(StrEqual(info, "sell"))
			{
				/* Forward: RP_OnSellItem */
				Call_StartForward(g_OnSellItem);
				Call_PushCell(client);
				Call_PushCell(itemID);
				Call_Finish();
				
				new hasitem = RP_HasPlayerInventoryItem(client, itemID);
				
				if(hasitem < durability)
				{
					if(durability >= 0)
						CPrintToChat(client, "%s You need %d more of this item in your inventory to sell it.", PL_PRE, durability-hasitem);
					else
						CPrintToChat(client, "%s You don't have this item in your inventory.", PL_PRE, durability, sItemName);
					invalid = true;
				}
				
				if(!invalid)
				{
					RP_SellPlayerItem(client, itemID, durability,iSell);
					CPrintToChat(client, "%s You have sold %dx %s.", PL_PRE, durability, sItemName);
				}
				
				Menu_ItemInfo(client);
			}
		}
	}
}

AddVendorItem(npcID,itemID,Float:multi)
{
	//Format(sQuery, sizeof(sQuery), "SELECT `vendorID`,`npcID`,`itemID`,`multiplier` FROM `rp_vendors`;");
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery),
	"INSERT INTO `rp_vendors` (npcID,itemID,multiplier) VALUES (%d,%d,%.2f)", npcID,itemID,multi);
	SQL_TQuery(g_hSQL, SQL_VendorAddItemCallback, sQuery, _, DBPrio_Normal);
	
	ResetVendors();
	LoadVendors();
}

public SQL_VendorAddItemCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_VendorAddItemCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

Float:GetVendorMultiplier(npcID, itemID)
{
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if (nNew[Vendor_NpcID] == npcID && nNew[Vendor_ItemID] == itemID)
		{
			return nNew[Vendor_Multiplier];
		}
	}
	
	return 1.0;
}

bool:HasNpcItem(npcID, itemID)
{
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if (nNew[Vendor_NpcID] == npcID && nNew[Vendor_ItemID] == itemID)
		{
			return true;
		}
	}
	
	return false;
}

/* Vendor Manager */

public Action:Cmd_VendorMgr(client, args)
{
	VendorManager(client);
	
	return Plugin_Handled;
}

VendorManager(client, npcID = -1)
{
	if(npcID == -1)
	{
		new entity = GetClientAimTarget(client, false);
		npcID = RP_GetNpcID(entity);
	}
	
	if(npcID <= 0)
		return;
	
	g_iSelectedNpcID[client] = npcID;
	
	if(RP_GetNpcType(npcID) != NT_Vendor)
		return;
	
	new iCount = RP_GetItemCount();
	
	new Handle:menu = CreateMenu(MenuHandler_VendorManager);
	SetMenuTitle(menu, "");
	
	for (new i = 0; i < iCount; i++)
	{
		new itemID = RP_GetItemID(i);
		
		decl String:sItemID[10];
		IntToString(itemID, sItemID, sizeof(sItemID));
		
		decl String:sItemName[64];
		RP_GetItemName(itemID, sItemName, sizeof(sItemName));
		
		new bool:hasitem = HasNpcItem(npcID, itemID);
		
		AddMenuItemEx(menu, false, sItemID, "[%d] %s Name: %s", itemID, hasitem ? "☑" : "☐", sItemName);
	}
	
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_VendorManager(Handle:menu, MenuAction:action, client, param2)
{
	if(0 >= client > MaxClients)
		return;
	
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new itemID = StringToInt(info);
			Menu_VendorManagerItemInfo(client, itemID);
		}
	}
}

Menu_VendorManagerItemInfo(client, itemID)
{
	g_iSelectedItemID[client] = itemID;
	
	decl String:sItemName[64];
	RP_GetItemName(itemID, sItemName, sizeof(sItemName));
	
	decl String:sItemDescription[64];
	RP_GetItemDescription(itemID, sItemDescription, sizeof(sItemDescription));
	
	new Handle:menu = CreateMenu(MenuHandler_VendorManagerItemInfo);
	SetMenuTitle(menu, "[%d] %s\nDescription: %s", itemID, sItemName, sItemDescription);
	
	AddMenuItemEx(menu, false, "manager", "Back to main menu");
	
	if(!HasNpcItem(g_iSelectedNpcID[client], itemID))
		AddMenuItemEx(menu, false, "enable", "Enable Item");
	else AddMenuItemEx(menu, false, "disable", "Disable Item");
	AddMenuItemEx(menu, false, "multi", "Adjust buy price");
	
	SetMenuExitButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_VendorManagerItemInfo(Handle:menu, MenuAction:action, client, param2)
{
	if (action == MenuAction_Select)
	{
		decl String:info[100], String:info2[100];
		new bool:found = GetMenuItem(menu, param2, info, sizeof(info), _, info2, sizeof(info2));
		
		if(found)
		{
			new itemID = g_iSelectedItemID[client];
			
			if(StrEqual(info, "manager"))
				VendorManager(client);
			else if(StrEqual(info, "enable"))
			{
				VendorEnableItem(client);
				Menu_VendorManagerItemInfo(client, itemID)
			}
			else if(StrEqual(info, "disable"))
			{
				VendorDisableItem(client);
				Menu_VendorManagerItemInfo(client, itemID)
			}
		}
	}
}

VendorEnableItem(client)
{
	new npcID = g_iSelectedNpcID[client];
	new itemID = g_iSelectedItemID[client];
	new Float:multiplier = 1.0;
	
	if(HasNpcItem(npcID, itemID))
		return;
	
	new nNew[RP_Vendor];
	nNew[Vendor_NpcID] = npcID;
	nNew[Vendor_ItemID] = itemID;
	nNew[Vendor_Multiplier] = multiplier;
	PushArrayArray(g_hVendors, nNew[0]);
	
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_vendors` (npcID, itemID, multiplier) VALUES (%d, %d, %.2f)", npcID, itemID, multiplier);
	SQL_TQuery(g_hSQL, SQL_InsertVendorCallback, sQuery, _);
}

public SQL_InsertVendorCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_InsertVendorCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

VendorDisableItem(client)
{
	new npcID = g_iSelectedNpcID[client];
	new itemID = g_iSelectedItemID[client];
	
	if(!DeleteVendorFromArray(npcID, itemID))
		return;
	
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "DELETE FROM `rp_vendors` WHERE npcID = %d AND itemID = %d", npcID, itemID);
	SQL_TQuery(g_hSQL, SQL_DeleteVendorCallback, sQuery, _);
}

bool:DeleteVendorFromArray(npcID, itemID)
{
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if (nNew[Vendor_NpcID] == npcID && nNew[Vendor_ItemID] == itemID)
		{
			RemoveFromArray(g_hVendors, i);
			return true;
		}
	}
	
	return false;
}

public SQL_DeleteVendorCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_InsertVendorCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public Native_AddVendorItem(Handle:plugin, numParams)
{
	AddVendorItem(GetNativeCell(1), GetNativeCell(2), GetNativeCell(3));
}

public Native_GetVendorCount(Handle:plugin, numParams)
{
	return GetArraySize(g_hVendors);
}

public Native_GetVendorIDByIndex(Handle:plugin, numParams)
{
	new index = GetNativeCell(1);
	new nNew[RP_Vendor];
	GetArrayArray(g_hVendors, index, nNew[0]);
	
	return nNew[Vendor_ID];
}

public Native_GetVendorIndexByVendorID(Handle:plugin, numParams)
{
	new vendorID = GetNativeCell(1);
	
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if(vendorID != nNew[Vendor_ID])
			return i;
	}
	
	return -1;
}

public Native_GetVendorNpcID(Handle:plugin, numParams)
{
	new vendorID = GetNativeCell(1);
	
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if(vendorID == nNew[Vendor_ID])
			return nNew[Vendor_NpcID];
	}
	
	return -1;
}

public Native_GetVendorMultiplier(Handle:plugin, numParams)
{
	new vendorID = GetNativeCell(1);
	
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if(vendorID == nNew[Vendor_ID])
		{
			SetNativeCellRef(2, nNew[Vendor_Multiplier]);
			break;
		}
	}
}

public Native_GetVendorItemCount(Handle:plugin, numParams)
{
	new vendorID = GetNativeCell(1);
	new iCount;
	
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if(vendorID != nNew[Vendor_ID])
			continue;
		
		iCount ++;
	}
	
	return iCount;
}

public Native_GetVendorItemTypeCount(Handle:plugin, numParams)
{
	new vendorID = GetNativeCell(1);
	new type = GetNativeCell(2);
	new iCount;
	
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if(vendorID != nNew[Vendor_ID])
			continue;
		
		if (RP_GetItemType(nNew[Vendor_ItemID]) == type)
			iCount ++;
	}
	
	return iCount;
}

public Native_GetVendorItemSubTypeCount(Handle:plugin, numParams)
{
	new vendorID = GetNativeCell(1);
	new type = GetNativeCell(2);
	new subType = GetNativeCell(3);
	new iCount;
	
	for (new i = 0; i < GetArraySize(g_hVendors); i++)
	{
		new nNew[RP_Vendor];
		GetArrayArray(g_hVendors, i, nNew[0]);
		
		if(vendorID != nNew[Vendor_ID])
			continue;
		
		if (RP_GetItemType(nNew[Vendor_ItemID]) == type && RP_GetItemSubType(nNew[Vendor_ItemID]) == subType)
			iCount ++;
	}
	
	return iCount;
}