#include "roleplay/roleplay"

#define PL_NAME		"RP_PLAYERS"

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	version = PL_VERSION,
	url = PL_URL
}

new bool:g_LateLoaded;

/* Database */
new Handle:g_hSQL;

/* Templates */
new nPlayerTeamplate[RP_Player];
new Handle:g_hPlayers = INVALID_HANDLE;

/* Forwards */
new Handle:g_OnPlayerReady;

new g_iPlayerIndex[MAXPLAYERS + 1] = {-1, ...};
new g_iPlayerID[MAXPLAYERS + 1] = {-1, ...};

new String:g_sAuth[MAXPLAYERS + 1][24];
new bool:g_bAuthed[MAXPLAYERS + 1];
new bool:g_iCreate[MAXPLAYERS + 1];

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	
	CreateNative("RP_GetPlayerID", Native_GetPlayerID);
	CreateNative("RP_GetClient", Native_GetClient);
	
	CreateNative("RP_GetName", Native_GetName);
	CreateNative("RP_GetSteamID", Native_GetSteamID);
	CreateNative("RP_GetTimeFirstSeen", Native_GetTimeFirstSeen);
	CreateNative("RP_GetTimeConnected", Native_GetTimeConnected);
	CreateNative("RP_GetTimeLastSeen", Native_GetTimeLastSeen);
	
	CreateNative("RP_GetNameByPlayerID", Native_GetNameByPlayerID);
	CreateNative("RP_GetSteamIDByPlayerID", Native_GetSteamIDByPlayerID);
	CreateNative("RP_GetTimeFirstSeenByPlayerID", Native_GetTimeFirstSeenByPlayerID);
	CreateNative("RP_GetTimeConnectedByPlayerID", Native_GetTimeConnectedByPlayerID);
	CreateNative("RP_GetTimeLastSeenByPlayerID", Native_GetTimeLastSeenByPlayerID);
	
	g_OnPlayerReady = CreateGlobalForward("RP_OnPlayerReady", ET_Event, Param_Cell, Param_Cell, Param_Cell);
	
	return APLRes_Success;
}

public OnPluginStart()
{
	/* Init ADT arrays */
	g_hPlayers = CreateArray(sizeof(nPlayerTeamplate));
	nPlayerTeamplate[Player_ID] = 0;
	
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

/* Create Table */
SQLConnectionReady()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_players` (`playerID` int(10) NOT NULL AUTO_INCREMENT, `last_name` varchar(64) NOT NULL, `steamID` varchar(32) NOT NULL, `communityID` varchar(32) NOT NULL, `first_seen` int(11) NOT NULL, `last_seen` int(11) NOT NULL, PRIMARY KEY (`playerID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
	
	Format(sQuery, sizeof(sQuery), "SELECT `playerID`, `steamID`, `last_name`,`first_seen`, `last_seen` FROM `rp_players`");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_LoadPlayersCallback, sQuery, _, DBPrio_High);
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public SQL_LoadPlayersCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadItemsCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	/* Fetch all items */
	while (SQL_FetchRow(hndl))
	{
		new nNew[RP_Player];
		
		nNew[Player_ID] = SQL_FetchInt(hndl, 0);
		SQL_FetchString(hndl, 1, nNew[Player_Auth], 32);
		SQL_FetchString(hndl, 2, nNew[Player_Name], 64);
		nNew[Player_TimeFirstSeen] = _:SQL_FetchInt(hndl, 3);
		nNew[Player_TimeLastSeen] = _:SQL_FetchInt(hndl, 4);
		
		nNew[Player_Client] = 0;
		nNew[Player_TimeConnected] = nNew[Player_TimeFirstSeen];
		
		PushArrayArray(g_hPlayers, nNew[0]);
	}
	
	/* Load players which are already connected */
	for(new i = 1; i <= MaxClients; i++)
	{
		ResetClient(i);
		
		if(!IsClientInGame(i))
			continue;
		
		if(IsFakeClient(i))
			continue;
		
		LoadClient(i);
	}
}

/* Reset Client OnClientDisconnect_Post */
public OnClientDisconnect_Post(client)
{
	if(IsClientConnected(client))
	{
		decl String:sName[MAX_NAME_LENGTH];
		decl String:sSafeName[((MAX_NAME_LENGTH * 2) + 1)];
		GetClientName(client, sName, sizeof(sName));
		SQL_EscapeString(g_hSQL, sName, sSafeName, sizeof(sSafeName));
	
		decl String:sQuery[2048];
		FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_players` SET last_name = '%s', last_seen = %d WHERE steamID = '%s'", sSafeName, GetTime(), g_sAuth[client]);
		SQL_TQuery(g_hSQL, SQL_ClientUpdateCallback, sQuery, _);
	}
	
	ResetClient(client);
}

/* Load Client OnClientPostAdminCheck */
public OnClientPostAdminCheck(client)
{
	ResetClient(client);
	
	if(IsFakeClient(client))
		return;

	LoadClient(client)
}

ResetClient(client)
{
	g_iCreate[client] = false;
	g_iPlayerIndex[client] = -1
	g_iPlayerID[client] = -1
}

LoadClient(client)
{
	g_bAuthed[client] = GetClientAuthString(client, g_sAuth[client], sizeof(g_sAuth[]));
	
	/* Player is not fully connected or database is down, try it again later */
	if(!g_bAuthed[client] || g_hSQL == INVALID_HANDLE)
	{
		g_bAuthed[client] = false;
		CreateTimer(1.0, Timer_AuthClient, GetClientUserId(client), TIMER_FLAG_NO_MAPCHANGE|TIMER_REPEAT);
		return;
	}

	decl String:sName[MAX_NAME_LENGTH];
	decl String:sSafeName[((MAX_NAME_LENGTH * 2) + 1)];
	GetClientName(client, sName, sizeof(sName));
	SQL_EscapeString(g_hSQL, sName, sSafeName, sizeof(sSafeName));
	
	/* Get cache index for player */
	for (new i = 0; i < GetArraySize(g_hPlayers); i++)
	{
		new nNew[RP_Player];
		GetArrayArray(g_hPlayers, i, nNew[0]);
		
		if (StrEqual(nNew[Player_Auth], g_sAuth[client]))
		{
			g_iPlayerIndex[client] = i;
			g_iPlayerID[client] = nNew[Player_ID];
			Format(nNew[Player_Name], 64, "%N", client);
			SetArrayArray(g_hPlayers, i, nNew[0]);
			break;
		}
	}
	
	new time = GetTime();
	
	if(g_iPlayerIndex[client] == -1)
	{
		g_iCreate[client] = true;
		decl String:sQuery[2048];
		FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_players` (steamID, last_name, first_seen, last_seen) VALUES ('%s', '%s', %d, %d)", g_sAuth[client], sSafeName, GetTime(), GetTime());
		SQL_TQuery(g_hSQL, SQL_ClientCreateCallback, sQuery, client, DBPrio_Normal);
		
		new nNew[RP_Player];
	
		FormatEx(nNew[Player_Auth], sizeof(nNew[Player_Auth]), "%s", g_sAuth[client]);
		FormatEx(nNew[Player_Name], sizeof(nNew[Player_Name]), "%s", sName);
		nNew[Player_Client] = client;
		nNew[Player_TimeFirstSeen] = time;
		nNew[Player_TimeConnected] = time;
		nNew[Player_TimeLastSeen] = time;
		
		g_iPlayerIndex[client] = GetArraySize(g_hPlayers);
			
		PushArrayArray(g_hPlayers, nNew[0]);
	}
	else
	{
		decl String:sQuery[2048];
		FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_players` SET last_name = '%s', last_seen = %d WHERE steamID = '%s'", sSafeName, time, g_sAuth[client]);
		SQL_TQuery(g_hSQL, SQL_ClientUpdateCallback, sQuery, _);
		
		new nNew[RP_Player];
		GetArrayArray(g_hPlayers, g_iPlayerIndex[client], nNew[0]);
		nNew[Player_TimeConnected] = time;
		nNew[Player_TimeLastSeen] = time;
		SetArrayArray(g_hPlayers, g_iPlayerIndex[client], nNew[0]);
	}
	
	/* Let other plugins know the player is ready */
	if(g_iCreate[client])
	{
		//Get the PlayerID
		LoadNewPlayer(client);
	}
	else
	{
		Call_StartForward(g_OnPlayerReady);
		Call_PushCell(client);
		Call_PushCell(g_iPlayerID[client]);
		Call_PushCell(false);
		Call_Finish();
	}
	g_iCreate[client] = false;
}

LoadNewPlayer(client)
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `playerID` FROM `rp_players` WHERE steamID = '%s'",g_sAuth[client]);
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_LoadNewPlayersCallback, sQuery, client, DBPrio_Normal);
}

public SQL_LoadNewPlayersCallback(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_LoadNewPlayerCallback) %s", PL_PRE_N, PL_NAME, error);
	}
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, g_iPlayerIndex[client], nNew[0]);
		
	if (StrEqual(nNew[Player_Auth], g_sAuth[client]))
	{
		if(SQL_FetchRow(hndl))
		{
			nNew[Player_ID] = SQL_FetchInt(hndl, 0);
			g_iPlayerID[client] = nNew[Player_ID];

			SetArrayArray(g_hPlayers, g_iPlayerIndex[client], nNew[0]);
				
			Call_StartForward(g_OnPlayerReady);
			Call_PushCell(client);
			Call_PushCell(g_iPlayerID[client]);
			Call_PushCell(true);
			Call_Finish();
		}
	}
}

/* For some reason we need to (re-)auth the player */
public Action:Timer_AuthClient(Handle:timer, any:userid)
{
	new client = GetClientOfUserId(userid);
	
	if(IsClientInGame(client))
		LoadClient(client);

	return Plugin_Stop;
}

public SQL_ClientCreateCallback(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientCreateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public SQL_ClientUpdateCallback(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientUpdateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

GetPlayerIndex(PlayerID)
{
	for (new i = 0; i < GetArraySize(g_hPlayers); i++)
	{
		new nNew[RP_Player];
		GetArrayArray(g_hPlayers, i, nNew[0]);
		
		if (nNew[Player_ID] == PlayerID)
		{
			return i;
		}
	}
	
	return -1;
}

/* Natives */

public Native_GetPlayerID(Handle:plugin, numParams)
{
	return g_iPlayerID[GetNativeCell(1)];
}

public Native_GetClient(Handle:plugin, numParams)
{
	new PlayerID = GetNativeCell(1);
	
	for(new i = 1; i <= MaxClients; i++)
	{
		if(!IsClientInGame(i))
			continue;
		
		if(IsFakeClient(i))
			continue;
		
		if(g_iPlayerID[i] == PlayerID)
			return i;
	}
	return -1
}

public Native_GetName(Handle:plugin, numParams)
{
	new index = g_iPlayerIndex[GetNativeCell(1)];
	if(index == -1)
		return -1;
	
	new nlen = GetNativeCell(3);
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	
	decl String:buffer[nlen];
	FormatEx(buffer, nlen, "%s", nNew[Player_Name]);
	if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
		return true;
	
	return false;
}

public Native_GetSteamID(Handle:plugin, numParams)
{
	new index = g_iPlayerIndex[GetNativeCell(1)];
	if(index == -1)
		return -1;
	
	new nlen = GetNativeCell(3);
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	
	decl String:buffer[nlen];
	FormatEx(buffer, nlen, "%s", nNew[Player_Auth]);
	if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
		return true;
	
	return false;
}

public Native_GetTimeFirstSeen(Handle:plugin, numParams)
{
	new index = g_iPlayerIndex[GetNativeCell(1)];
	if(index == -1)
		return -1;
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	return nNew[Player_TimeFirstSeen];
}

public Native_GetTimeLastSeen(Handle:plugin, numParams)
{
	new index = g_iPlayerIndex[GetNativeCell(1)];
	if(index == -1)
		return -1;
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	return nNew[Player_TimeLastSeen];
}

public Native_GetTimeConnected(Handle:plugin, numParams)
{
	new index = g_iPlayerIndex[GetNativeCell(1)];
	if(index == -1)
		return -1;
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	return nNew[Player_TimeConnected];
}

public Native_GetNameByPlayerID(Handle:plugin, numParams)
{
	new index = GetPlayerIndex(GetNativeCell(1));
	if(index == -1)
		return -1;
	
	new nlen = GetNativeCell(3);
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	
	decl String:buffer[nlen];
	FormatEx(buffer, nlen, "%s", nNew[Player_Name]);
	if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
		return true;
	
	return false;
}

public Native_GetSteamIDByPlayerID(Handle:plugin, numParams)
{
	new index = GetPlayerIndex(GetNativeCell(1));
	if(index == -1)
		return -1;
	
	new nlen = GetNativeCell(3);
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	
	decl String:buffer[nlen];
	FormatEx(buffer, nlen, "%s", nNew[Player_Auth]);
	if (SetNativeString(2, buffer, nlen, true) == SP_ERROR_NONE)
		return true;
	
	return false;
}

public Native_GetTimeFirstSeenByPlayerID(Handle:plugin, numParams)
{
	new index = GetPlayerIndex(GetNativeCell(1));
	if(index == -1)
		return -1;
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	return nNew[Player_TimeFirstSeen];
}

public Native_GetTimeLastSeenByPlayerID(Handle:plugin, numParams)
{
	new index = GetPlayerIndex(GetNativeCell(1));
	if(index == -1)
		return -1;
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	return nNew[Player_TimeLastSeen];
}

public Native_GetTimeConnectedByPlayerID(Handle:plugin, numParams)
{
	new index = GetPlayerIndex(GetNativeCell(1));
	if(index == -1)
		return -1;
	
	new nNew[RP_Player];
	GetArrayArray(g_hPlayers, index, nNew[0]);
	return nNew[Player_TimeConnected];
}