#include "roleplay/roleplay"
#include <sourcemod>

#define PL_NAME		"RP_TEAMS"
#define RESPAWNTIME	5.0

new bool:g_LateLoaded;
new bool:g_CopMode[MAXPLAYERS+1];
new bool:g_CopRights[MAXPLAYERS+1];

new bool:g_respawned[MAXPLAYERS+1];

/* Database */
new Handle:g_hSQL;

public Plugin:myinfo = 
{
	name = PL_NAME,
	author = PL_AUTHOR,
	description = "",
	version = PL_VERSION,
	url = PL_URL
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	g_LateLoaded = late;
	RegPluginLibrary(PL_NAME);
	CreateNative("RP_SetCop", Native_SetCop);
	CreateNative("RP_IsCop", Native_IsCop);
	CreateNative("RP_ChangeCopMode", Native_ChangeCopMode);
	return APLRes_Success;
}

public OnPluginStart()
{
	if(g_LateLoaded)
	{
		g_hSQL = Handle:RP_GetSql();
		
		if(g_hSQL != INVALID_HANDLE)
			SQLConnectionReady();
		
		g_LateLoaded = false;
	}
	
	RegConsoleCmd("kill", InterceptKill);
	RegConsoleCmd("jointeam", InterceptKill);
	RegConsoleCmd("spectate", InterceptKill);
	RegConsoleCmd("sm_switch", Command_Switch);
	
	HookEvent("player_team", Event_Spawn);
	HookEvent("player_death", Event_Death);
}

public OnPluginEnd()
{
	UnhookEvent("player_team", Event_Spawn);
	UnhookEvent("player_death", Event_Death);
}

public Event_Spawn( Handle:Spawn_Event, const String:Death_Name[], bool:Death_Broadcast )
{
	new client = GetClientOfUserId( GetEventInt(Spawn_Event,"userid") );
	new team = GetEventInt(Spawn_Event, "team");

	if( client != 0 )
	{
		if((team == CS_TEAM_CT || team == CS_TEAM_SPECTATOR) && (!g_CopMode[client] || !g_CopRights[client]))
		{
			if(team == CS_TEAM_SPECTATOR)
				PrintToChat(client, "[Spawn] No Spectators allowed. Switching to Civilian");
			if(!g_CopRights[client])
				PrintToChat(client, "[Spawn] You got no Coprights. Switching to Civilian");			
			else if(!g_CopMode[client])
				PrintToChat(client, "[Spawn] You are not in CopMode. Switching to Civilian");
					
			ChangeClientTeam(client, CS_TEAM_T);
			CreateTimer(RESPAWNTIME, RespawnClient, any:client);
		}
		else
		{
			CreateTimer(0.5, SetUpClientSpawn, any:client);
		}
	}
}

public Action:SetUpClientSpawn( Handle:timer, any:client )
{
	if(RP_IsClientCuffed(client))
		RP_CuffClient(client);
	RP_CheckClientJailStatus(client);
}

public Event_Death( Handle:Death_Event, const String:Death_Name[], bool:Death_Broadcast )
{
	new client = GetClientOfUserId( GetEventInt(Death_Event,"userid") );
	new team = GetClientTeam(client);
	
	if( client != 0 )
	{
		if((team == CS_TEAM_CT || team == CS_TEAM_SPECTATOR) && (!g_CopMode[client] || !g_CopRights[client]))
		{					
			ChangeClientTeam(client, CS_TEAM_T);
			CreateTimer(RESPAWNTIME, RespawnClient, any:client);
		}
		else
		{
			CreateTimer(RESPAWNTIME, RespawnClient, any:client);
		}
	}
}

public Action:RespawnClient( Handle:timer, any:client )
{
	if ( IsValidEntity(client) && IsClientInGame(client) && IsClientObserver(client) && !IsPlayerAlive(client) )
	{
  		new ragdoll = GetEntPropEnt(client, Prop_Send, "m_hRagdoll");
  		if ( ragdoll > 0 )
		{
			new ent = CreateEntityByName("env_entity_dissolver");
			if ( ent > 0 )
			{
				new String:dissolvename[32];
				Format(dissolvename, sizeof(dissolvename), "dis_%d", client);
				DispatchKeyValue(ragdoll, "targetname", dissolvename);
				DispatchKeyValue(ent, "dissolvetype", "0");
				DispatchKeyValue(ent, "target", dissolvename);
				AcceptEntityInput(ent, "Dissolve");
				AcceptEntityInput(ent, "kill");
			}
		}			
		CS_RespawnPlayer(client);
		CreateTimer(0.5, SetUpClientSpawn, client);
	}
}

public Action:Command_Switch(client, args)
{
	if(0 >= client > MaxClients)
		return Plugin_Handled;
	
	if(!IsClientInGame(client))
		return Plugin_Handled;
	
	if(!IsPlayerAlive(client))
		return Plugin_Handled;
	
	if(g_CopRights[client] && RP_GetCrime(client) <= 0 && !RP_IsClientCuffed(client))
	{
		RP_ChangeCopMode(client);
		CreateTimer(1.0, RespawnClient, client);
	}
	else
	{
		if(!g_CopRights[client])
			CPrintToChat(client, "%s You are no cop.", PL_PRE);
		else
			CPrintToChat(client, "%s Criminals can't be cops.", PL_PRE);
	}
	return Plugin_Handled;
}

public OnMapStart()
{
	ServerCommand("mp_teamname_1 Cops");
	ServerCommand("mp_teamname_2 Civilian");
}

public RP_OnSqlConnected(Handle:db)
{
	g_hSQL = db;
	SQLConnectionReady();
}

public RP_OnSqlStop()
{
	g_hSQL = INVALID_HANDLE;
}

public RP_OnPlayerReady(client, playerID, bool:create)
{
	if(create)
	{		
		decl String:sQuery[2048];
		FormatEx(sQuery, sizeof(sQuery), "INSERT INTO `rp_player_cop` (playerID, isCop) VALUES (%d, 0)", RP_GetPlayerID(client));
		SQL_TQuery(g_hSQL, SQL_ClientCreateCallback, sQuery, _, DBPrio_Normal);
		
		SetPlayerDefaults(client);
	}
	else
	{
		LoadPlayer(client);
	}
}

public SQL_ClientCreateCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientCreateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

LoadPlayer(client)
{
	g_CopMode[client] = false;
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "SELECT `isCop` FROM `rp_player_cop` WHERE `playerID` = %d", RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientLoadCallback, sQuery, GetClientUserId(client), DBPrio_Low);
}

public SQL_ClientLoadCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientConnectCallback) %s", PL_PRE_N, PL_NAME, error);
	}

	new client = GetClientOfUserId(userid);
	if(!client || !IsClientInGame(client))
		return;
	
	if(SQL_GetRowCount(hndl))
	{
		SQL_FetchRow(hndl);
		g_CopRights[client] = bool:SQL_FetchInt(hndl, 0);
	}
}

SetPlayerDefaults(client)
{
	g_CopRights[client] = false;
	g_CopMode[client] = false;
}

SQLConnectionReady()
{
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "CREATE TABLE IF NOT EXISTS `rp_player_cop` (`playerID` int(10) NOT NULL, `isCop` BOOL NOT NULL DEFAULT '0', PRIMARY KEY (`playerID`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
	LogMessage("%s [%s] Query: %s", PL_PRE_N, PL_NAME, sQuery);
	SQL_TQuery(g_hSQL, SQL_CreateTableCallback, sQuery, _, DBPrio_High);
}

public SQL_CreateTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_CreateTableCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

public OnClientConnected(client)
{
	g_CopMode[client] = false;
	g_respawned[client] = false;
}

public Action:InterceptKill(client, args)
{
	if(!IsClientInGame(client))
		return Plugin_Continue;
	
	if(!IsPlayerAlive(client))
		return Plugin_Continue;
	
	new team = GetClientTeam(client);
	
	//Ignore players in spec
	if(team < CS_TEAM_T)
		return Plugin_Continue;
	
	//Just double check
	if(team == CS_TEAM_CT && !g_CopMode[client] && !g_CopRights[client])
	{
		ChangeClientTeam(client , CS_TEAM_T);
		CreateTimer(RESPAWNTIME, RespawnClient, any:client);
		return Plugin_Continue;
	}
	
	if(RP_GetCrime(client) == 0 || !RP_IsClientCuffed(client) /*|| !knockedout*/)
	{
		//Check player is moving to abort?
		PrintToChat(client, "%s You'll die in 10 seconds [Anti Escape]", PL_PRE);
		CreateTimer(10.0, SuicideTimer, any:client);
		return Plugin_Handled;
	}
	else
	{
		CPrintToChat(client, "%s You can't kill yourself right now!", PL_PRE);
		return Plugin_Handled;
	}
}

public Action:SuicideTimer(Handle:timer, any:client)
{
	if(IsClientInGame(client) && IsPlayerAlive(client))
	{
		ForcePlayerSuicide(client);
		return Plugin_Continue;
	}
	return Plugin_Continue;
}

public IsCop(client)
{
	return (g_CopMode[client] && g_CopRights[client]);
}

public SetCop(client,bool:set)
{
	if(set)
	{
		g_CopRights[client] = true;
		CPrintToChat(client, "%s You have Coprights now", PL_PRE);
		SavePlayer(client);
	}
	else
	{
		g_CopRights[client] = false;
		CPrintToChat(client, "%s You have no Coprights anymore", PL_PRE);
		if(GetClientTeam(client) == CS_TEAM_CT || g_CopMode[client] == true)
		{
			ChangeClientTeam(client , CS_TEAM_T);
			CreateTimer(1.0, RespawnClient, any:client);
		}
		SavePlayer(client);
	}
}

SavePlayer(client)
{
	decl String:sQuery[2048];
	FormatEx(sQuery, sizeof(sQuery), "UPDATE `rp_player_cop` SET isCop = %d WHERE playerID = %d", g_CopRights[client], RP_GetPlayerID(client));
	SQL_TQuery(g_hSQL, SQL_ClientUpdateCallback, sQuery, _);
}

public SQL_ClientUpdateCallback(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	if (owner == INVALID_HANDLE || hndl == INVALID_HANDLE)
	{
		LogError("%s [%s] (SQL_ClientUpdateCallback) %s", PL_PRE_N, PL_NAME, error);
	}
}

ChangeCopMode(client)
{
	//Cop not in Copmode
	if(!g_CopMode[client] && g_CopRights[client])
	{
		g_CopMode[client] = true;
		ChangeClientTeam(client , CS_TEAM_CT);
		CreateTimer(1.0, RespawnClient, any:client);
	}
	//Cop in Copmode
	else if(g_CopMode[client] && g_CopRights[client])
	{
		g_CopMode[client] = false;
		ChangeClientTeam(client , CS_TEAM_T);
		CreateTimer(1.0, RespawnClient, any:client);
	}
	//No Cop
	else
	{
		CPrintToChat(client, "%s You have no Coprights", PL_PRE);
	}
}

public Native_SetCop(Handle:plugin, numParams)
{
	SetCop(GetNativeCell(1), GetNativeCell(2));
}

public Native_IsCop(Handle:plugin, numParams)
{
	return IsCop(GetNativeCell(1));
}

public Native_ChangeCopMode(Handle:plugin, numParams)
{
	ChangeCopMode(GetNativeCell(1));
}